package cn.iocoder.yudao.module.wealth.dal.dataobject.receivable;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;

import java.math.BigDecimal;

/**
 * 应收款 DO
 *
 * @author 郑屹
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@TableName("ecw_receivable")
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class ReceivableDO extends BaseDO {

    /**
     * 主键
     */
    @TableId
    private Long id;
    /**
     * 收款单id
     */
    private Long receiptId;
    /**
     * 收款单号
     */
    private String receiptNo;
    /**
     * 订单id
     */
    private Long orderId;
    /**
     * 订单号
     */
    private String orderNo;
    /**
     * 订单商品ID
     */
    private Long orderItemId;
    /**
     * 商品中文名
     */
    private String titleZh;
    /**
     * 商品英文名
     */
    private String titleEn;
    /**
     * 数量
     */
    private Integer quantity;
    /**
     * 箱数
     */
    private Integer num;
    /**
     * 体积/重量
     */
    private BigDecimal weight;
    /**
     * 体积
     */
    private BigDecimal volume;
    /**
     * 收款类型(字典 payment_type)
     */
    private Integer collectionType;
    /**
     * 费用类型(字典 receivable_fee_type)
     */
    private Integer feeType;
    /**
     * 汇率
     */
    private BigDecimal exchangeRate;
    /**
     * 货币id
     */
    private Long currencyId;
    /**
     * 单价
     */
    private BigDecimal unitPrice;
    /**
     * 总金额
     */
    private BigDecimal totalAmount;
    /**
     * 优惠金额
     */
    private BigDecimal discountTotal;

    /**
     * 货值
     */
    private BigDecimal amountWorth;

    /**
     * 优惠备注
     */
    private String discountRemark;

    /**
     * 是否预付：0 到付 1 预付
     */
    private Integer isPayAdvance;
    /**
     * 收款状态（0未收款，1收款中，2已收款）
     */
    private Integer state;
    /**
     * 数据来源（pc端，mobile端）
     */
    private String source;
    /**
     * 作者
     */
    private String author;

    /**
     * 申请人姓名
     */
    @TableField(exist = false)
    @ApiModelProperty(value = "申请人姓名")
    private String authorName;

    @TableField(exist = false)
    @ApiModelProperty(value = "核销比例要除以100")
    private String writeOffProportion;

    @ApiModelProperty(value = "付款人")
    private String paymentUser;

    @ApiModelProperty(value = "费用来源：1 订单计算 2 费用申请 3 特需费用")
    private Integer feeSource;
    /**
     * 税点
     */
    private BigDecimal taxPoint;
    /**
     * 含税金额
     */
    private BigDecimal taxAmount;

    /**
     * 备注
     */
    private String remark;

}
