package cn.iocoder.yudao.module.wealth.dal.mysql.receiptAccount;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountQueryVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.math.BigDecimal;
import java.util.List;

/**
 * 收款单收款账户 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface ReceiptAccountMapper extends AbstractMapper<ReceiptAccountDO> {
    @Override
    default PageResult<ReceiptAccountDO> selectPage(PageVO page, Object object) {
        if (object instanceof ReceiptAccountQueryVO) {
            ReceiptAccountQueryVO vo = (ReceiptAccountQueryVO) object;
            return selectPage(page, new LambdaQuery<ReceiptAccountDO>()
                    .eqIfPresent(ReceiptAccountDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceiptAccountDO::getCurrencyId, vo.getCurrencyId())
                    .eqIfPresent(ReceiptAccountDO::getPlatformAccountId, vo.getPlatformAccountId())
                    .eqIfPresent(ReceiptAccountDO::getPlatformAccountNo, vo.getPlatformAccountNo())
                    .likeIfPresent(ReceiptAccountDO::getPlatformAccountName, vo.getPlatformAccountName())
                    .betweenIfPresent(ReceiptAccountDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ReceiptAccountDO::getId));
        }

        return null;
    }

    @Override
    default List<ReceiptAccountDO> selectList(Object object) {
        if (object instanceof ReceiptAccountQueryVO) {
            ReceiptAccountQueryVO vo = (ReceiptAccountQueryVO) object;
            return selectList(new LambdaQuery<ReceiptAccountDO>()
                    .eqIfPresent(ReceiptAccountDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceiptAccountDO::getCurrencyId, vo.getCurrencyId())
                    .eqIfPresent(ReceiptAccountDO::getPlatformAccountId, vo.getPlatformAccountId())
                    .eqIfPresent(ReceiptAccountDO::getPlatformAccountNo, vo.getPlatformAccountNo())
                    .likeIfPresent(ReceiptAccountDO::getPlatformAccountName, vo.getPlatformAccountName())
                    .betweenIfPresent(ReceiptAccountDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ReceiptAccountDO::getId));
        }

        return null;
    }

    /*
     获取核销基准金额
     */
    @ResultType(BigDecimal.class)
    @Select({
            "<script>",
            "select COALESCE(sum(r.write_off_amount),0) ",
            "from ecw_receipt_account r ",
            "where r.deleted = 0",
            "AND r.receipt_id =  #{receiptId} ",
            "</script>"
    })
    BigDecimal getWriteOffAmountByReceiptId(@Param("receiptId") Long receiptId);
}
