package cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemQueryVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.math.BigDecimal;
import java.util.List;

/**
 * 收款单明细 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface ReceiptItemMapper extends AbstractMapper<ReceiptItemDO> {
    @Override
    default PageResult<ReceiptItemDO> selectPage(PageVO page, Object object) {
        if (object instanceof ReceiptItemQueryVO) {
            ReceiptItemQueryVO vo = (ReceiptItemQueryVO) object;
            return selectPage(page, new LambdaQuery<ReceiptItemDO>()
                    .eqIfPresent(ReceiptItemDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceiptItemDO::getAccountNo, vo.getAccountNo())
                    .eqIfPresent(ReceiptItemDO::getAmount, vo.getAmount())
                    .likeIfPresent(ReceiptItemDO::getAccountName, vo.getAccountName())
                    .eqIfPresent(ReceiptItemDO::getCurrencyId, vo.getCurrencyId())
                    .betweenIfPresent(ReceiptItemDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .eqIfPresent(ReceiptItemDO::getRate, vo.getRate())
                    .betweenIfPresent(ReceiptItemDO::getAmountDate, vo.getBeginAmountDate(), vo.getEndAmountDate())
                    .eqIfPresent(ReceiptItemDO::getBillNo, vo.getBillNo())
                    .eqIfPresent(ReceiptItemDO::getAttr, vo.getAttr())
                    .eqIfPresent(ReceiptItemDO::getStatus, vo.getStatus())
                    .eqIfPresent(ReceiptItemDO::getWriteOffCurrencyId, vo.getWriteOffCurrencyId())
                    .eqIfPresent(ReceiptItemDO::getWriteOffAmount, vo.getWriteOffAmount())
                    .orderByDesc(ReceiptItemDO::getId));
        }

        return null;
    }

    @Override
    default List<ReceiptItemDO> selectList(Object object) {
        if (object instanceof ReceiptItemQueryVO) {
            ReceiptItemQueryVO vo = (ReceiptItemQueryVO) object;
            return selectList(new LambdaQuery<ReceiptItemDO>()
                    .eqIfPresent(ReceiptItemDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceiptItemDO::getAccountNo, vo.getAccountNo())
                    .eqIfPresent(ReceiptItemDO::getAmount, vo.getAmount())
                    .likeIfPresent(ReceiptItemDO::getAccountName, vo.getAccountName())
                    .eqIfPresent(ReceiptItemDO::getCurrencyId, vo.getCurrencyId())
                    .betweenIfPresent(ReceiptItemDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .eqIfPresent(ReceiptItemDO::getRate, vo.getRate())
                    .betweenIfPresent(ReceiptItemDO::getAmountDate, vo.getBeginAmountDate(), vo.getEndAmountDate())
                    .eqIfPresent(ReceiptItemDO::getBillNo, vo.getBillNo())
                    .eqIfPresent(ReceiptItemDO::getAttr, vo.getAttr())
                    .eqIfPresent(ReceiptItemDO::getStatus, vo.getStatus())
                    .eqIfPresent(ReceiptItemDO::getWriteOffCurrencyId, vo.getWriteOffCurrencyId())
                    .eqIfPresent(ReceiptItemDO::getWriteOffAmount, vo.getWriteOffAmount())
                    .orderByDesc(ReceiptItemDO::getId));
        }

        return null;
    }

    /*
     获取已核销金额
     */
    @ResultType(BigDecimal.class)
    @Select({
            "<script>",
            "select COALESCE(sum(r.write_off_amount),0) ",
            "from ecw_receipt_item r ",
            "where r.deleted = 0",
            "AND r.status = 1",
            "AND r.receipt_id =  #{receiptId} ",
            "</script>"
    })
    BigDecimal getWriteOffAmountByReceiptId(@Param("receiptId") Long receiptId);
}
