package cn.iocoder.yudao.module.wealth.service;

import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.bpm.resp.TaskResp;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayable.CommissionPayableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayment.CommissionPaymentDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payment.PaymentDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.paymentApproval.PaymentApprovalDO;
import cn.iocoder.yudao.module.wealth.enums.PaymentStatusEnum;
import cn.iocoder.yudao.module.wealth.service.commissionPayable.CommissionPayableService;
import cn.iocoder.yudao.module.wealth.service.commissionPayment.CommissionPaymentService;
import cn.iocoder.yudao.module.wealth.service.payment.PaymentService;
import cn.iocoder.yudao.module.wealth.service.paymentApproval.PaymentApprovalService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

@Service
@Slf4j
public class FinancePaymentApproveService {

    @Autowired
    PaymentService paymentService;

    @Autowired
    CommissionPaymentService commissionPaymentService;
    @Autowired
    private CommissionPayableService commissionPayableService;
    @Autowired
    BpmCreateServiceFactory bpmCreateServiceFactory;

    @Autowired
    PaymentApprovalService paymentApprovalService;

    @Transactional
    public void financeApproveCallBack(String bmpKey, String businessKey, Integer result) {
        PaymentApprovalDO paymentApprovalDO = paymentApprovalService.getPaymentApproval(Long.parseLong(businessKey));
        WorkFlowEmus workFlowEmus = WorkFlowEmus.keyOf(bmpKey);
        if (Objects.isNull(workFlowEmus)) {
            throw exception(BMP_NOT_EXISTS);
        }
        Integer state = null;
        // 查询最后审核的详细信息

        TaskResp taskResp = bpmCreateServiceFactory.getLastTaskByProcessInstanceId(paymentApprovalDO.getBmpId());
        String comment = "";
        if (Objects.nonNull(taskResp)) {
            comment = Objects.isNull(taskResp.getComment()) ? "" : taskResp.getComment();
        }
        //根据不同的流程实例进行处理
        switch (workFlowEmus) {
            case FINANCE_PAYMENT_APPROVE:
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.APPROVE_FAIL.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.DRAFT.getValue();
                }
                updatePaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            case FINANCE_PAYMENT_APPROVE_NO:
                //付款单反审核
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.DRAFT.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                }
                this.updatePaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            case FINANCE_PAYMENT_WRITE_OFF:
                //付款单核销审核
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                }
                this.updatePaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            case FINANCE_PAYMENT_WRITE_OFF_NO:
                //付款单核销反审核
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF.getValue();
                }
                this.updatePaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            case FINANCE_COMMISSION_PAYMENT_APPROVE:
                //佣金付款单审核
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.APPROVE_FAIL.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.DRAFT.getValue();
                }
                this.updateCommissionPaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            case FINANCE_COMMISSION_PAYMENT_APPROVE_NO:
                //佣金付款单反审核
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.DRAFT.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                }
                this.updateCommissionPaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            case FINANCE_COMMISSION_PAYMENT_WRITE_OFF:
                //佣金付款单核销审核
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                }
                this.updateCommissionPaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            case FINANCE_COMMISSION_PAYMENT_WRITE_OFF_NO:
                //佣金付款单核销反审核
                if (null == paymentApprovalDO.getPaymentId()) {
                    throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
                }
                if (Objects.equals(result, BpmProcessInstanceResultEnum.APPROVE.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF_WAITING.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.REJECT.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF.getValue();
                } else if (Objects.equals(result, BpmProcessInstanceResultEnum.CANCEL.getResult())) {
                    state = PaymentStatusEnum.WRITE_OFF.getValue();
                }
                this.updateCommissionPaymentApproval(result, paymentApprovalDO, state, comment);
                break;
            default:
                throw exception(BMP_NOT_EXISTS);
        }


        PaymentApprovalDO updatePaymentApproval = new PaymentApprovalDO();
        updatePaymentApproval.setId(paymentApprovalDO.getId());
        updatePaymentApproval.setStatus(result);
        updatePaymentApproval.setProcessingResults(comment);
        paymentApprovalService.updateById(updatePaymentApproval);
    }

    private void updatePaymentApproval(Integer result, PaymentApprovalDO paymentApprovalDO, Integer state, String comment) {
        PaymentDO paymentDO = paymentService.getPayment(paymentApprovalDO.getPaymentId());
        if (null == paymentDO) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        PaymentDO updateReceipt = new PaymentDO();
        updateReceipt.setId(paymentDO.getId());
        updateReceipt.setBmpStatus(result);
        updateReceipt.setState(state);
        updateReceipt.setComment(comment);
        paymentService.updateById(updateReceipt);
    }

    private void updateCommissionPaymentApproval(Integer result, PaymentApprovalDO paymentApprovalDO, Integer state, String comment) {
        CommissionPaymentDO commissionPaymentDO = commissionPaymentService.getCommissionPayment(paymentApprovalDO.getPaymentId());
        if (null == commissionPaymentDO) {
            throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
        }
        CommissionPaymentDO updateReceipt = new CommissionPaymentDO();
        updateReceipt.setId(commissionPaymentDO.getId());
        updateReceipt.setBmpStatus(result);
        updateReceipt.setState(state);
        updateReceipt.setComment(comment);
        commissionPaymentService.updateById(updateReceipt);
        if (Objects.equals(PaymentStatusEnum.WRITE_OFF.getValue(), state)) {
            // 已核销，则更新关联佣金付款明细状态为已付款
            commissionPayableService.update(new LambdaUpdateWrapper<CommissionPayableDO>()
                    .set(CommissionPayableDO::getState, 2)
                    .eq(CommissionPayableDO::getPaymentId, updateReceipt.getId()));
        } else {
            // 未核销，则更新关联佣金付款明细状态为付款中(只要佣金应付明细关联有付款单，则表示付款中)
            commissionPayableService.update(new LambdaUpdateWrapper<CommissionPayableDO>()
                    .set(CommissionPayableDO::getState, 1)
                    .eq(CommissionPayableDO::getPaymentId, updateReceipt.getId()));
        }
    }
}
