package cn.iocoder.yudao.module.wealth.service;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.bpm.resp.TaskResp;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receipt.ReceiptMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.enums.ReceiptItemStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.ReceiptStatusEnum;
import cn.iocoder.yudao.module.wealth.service.receiptApproval.ReceiptApprovalService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIPT_ITEM_NOT_FOUND;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIPT_NOT_FOUND;

@Service
@Slf4j
public class FinanceReceiptApproveService {

    @Autowired
    private ReceiptMapper receiptMapper;

    @Resource
    private ReceiptItemMapper receiptItemMapper;

    @Autowired
    BpmCreateServiceFactory bpmCreateServiceFactory;

    @Autowired
    ReceiptApprovalService receiptApprovalService;


    @Resource
    ReceivableService receivableService;

    @Transactional
    public void financeApproveCallBack(String bmpKey, String businessKey, Integer result) {
        ReceiptApprovalDO receiptApprovalDO = receiptApprovalService.getReceiptApproval(Long.parseLong(businessKey));
        // 查询最后审核的详细信息
        TaskResp taskResp =
                bpmCreateServiceFactory.getLastTaskByProcessInstanceId(
                        receiptApprovalDO.getBmpId());
        String comment = "";
        if (Objects.nonNull(taskResp)) {
            comment = taskResp.getComment();
        }
        //根据不同的流程实例进行处理
        if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_APPROVE.getKey(), bmpKey)) {
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO =
                    receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            //添加订单ID的修改，不然订单ID就丢失了
            updateReceipt.setOrderId(receiptDO.getOrderId());

            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.RECEIPT_WAITING.getValue());

                LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                List<ReceivableDO> orderList = receivableService.list(lambdaQueryWrapper);
                List orderIds = orderList.stream().map(ReceivableDO::getOrderId).distinct().collect(Collectors.toList());
                receivableService.orderReceivableAddReceiptLog(orderIds, receiptDO.getReceiptNo());
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.APPROVE_FAIL.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.DRAFT.getValue());
            }
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);

        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_APPROVE_NO.getKey(), bmpKey)) {
            //收款单反审核
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO = receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            // 添加订单ID的修改，不然订单ID就丢失了
            updateReceipt.setOrderId(receiptDO.getOrderId());
            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.DRAFT.getValue());
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.RECEIPT_WAITING.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.RECEIPT_WAITING.getValue());
            }
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF.getKey(), bmpKey)) {
            //收款单核销审核
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO = receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            // 添加订单ID的修改，不然订单ID就丢失了
            updateReceipt.setOrderId(receiptDO.getOrderId());
            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                //需要根据是否要开票变成已核销，还是已核销待开票
                if (receiptDO.getOpenInvoice() == 0) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue());
                }
                if (receiptDO.getOpenInvoice() == 1) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_OPENING.getValue());
                }

                //更新收该收款单下的应收为收款完成状态
                LambdaUpdateWrapper<ReceivableDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
                lambdaUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                lambdaUpdateWrapper.set(ReceivableDO::getState, 2);
                receivableService.update(lambdaUpdateWrapper);

                updateReceipt.setWriteOffAt(new Date());
                updateReceipt.setWriteOffProportion(new BigDecimal("100"));//设置核销比例

                //添加订单核销日志
                LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                List<ReceivableDO> orderList = receivableService.list(lambdaQueryWrapper);
                List orderIds = orderList.stream().map(ReceivableDO::getOrderId).distinct().collect(Collectors.toList());

                receivableService.orderReceivableWriteOffLogAdd(orderIds, receiptDO.getReceiptNo());

            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue());
            }
            updateReceipt.setPayedAt(new Date());
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_WRITE_OFF_NO.getKey(), bmpKey)) {
            //收款单核销反审核
            if (null == receiptApprovalDO.getReceiptId()) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO receiptDO = receiptMapper.selectById(receiptApprovalDO.getReceiptId());
            if (null == receiptDO) {
                throw exception(RECEIPT_NOT_FOUND);
            }
            ReceiptDO updateReceipt = new ReceiptDO();
            // 添加订单ID的修改，不然订单ID就丢失了
            updateReceipt.setOrderId(receiptDO.getOrderId());
            updateReceipt.setId(receiptDO.getId());
            updateReceipt.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                //需要根据应收明细核销状态 变更成部分核销或者待核销
                LambdaQueryWrapper<ReceiptItemDO> receiptItemQueryWrapper = new LambdaQueryWrapper();
                receiptItemQueryWrapper.eq(ReceiptItemDO::getReceiptId, receiptDO.getId());
                receiptItemQueryWrapper.eq(ReceiptItemDO::getStatus, ReceiptItemStatusEnum.WRITE_OFF.getValue());
                if (receiptItemMapper.selectCount(receiptItemQueryWrapper) == 0) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue());
                } else {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
                }

                //更新收该收款单下的应收为收款完成状态
                LambdaUpdateWrapper<ReceivableDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
                lambdaUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                lambdaUpdateWrapper.set(ReceivableDO::getState, 1);
                receivableService.update(lambdaUpdateWrapper);

                //添加订单反核销日志
                LambdaQueryWrapper<ReceivableDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                lambdaQueryWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                List<ReceivableDO> orderList = receivableService.list(lambdaQueryWrapper);
                List orderIds = orderList.stream().map(ReceivableDO::getOrderId).distinct().collect(Collectors.toList());
                receivableService.orderReceivableCancelWriteOffLogAdd(orderIds, receiptDO.getReceiptNo());
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                //需要根据是否要开票变成已核销，还是已核销待开票
                if (receiptDO.getOpenInvoice() == 0) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue());
                }
                if (receiptDO.getOpenInvoice() == 1) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_OPENING.getValue());
                }
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                //需要根据是否要开票变成已核销，还是已核销待开票
                if (receiptDO.getOpenInvoice() == 0) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue());
                }
                if (receiptDO.getOpenInvoice() == 1) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_ALL_OPENING.getValue());
                }
            }
            updateReceipt.setComment(comment);
            receiptMapper.updateById(updateReceipt);
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getKey(), bmpKey)) {
            //收款单明细核销审核
            if (null == receiptApprovalDO.getReceiptItemId()) {
                throw exception(RECEIPT_ITEM_NOT_FOUND);
            }
            ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(receiptApprovalDO.getReceiptItemId());
            if (null == receiptItemDO) {
                throw exception(RECEIPT_ITEM_NOT_FOUND);
            }
            ReceiptItemDO updateReceiptItem = new ReceiptItemDO();
            updateReceiptItem.setId(receiptItemDO.getId());
            updateReceiptItem.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateReceiptItem.setStatus(ReceiptItemStatusEnum.WRITE_OFF.getValue());
                //判断收款单状态如果是待核销状态-变成部分核销状态，其它 状态不变
                ReceiptDO receiptDO = receiptMapper.selectById(receiptItemDO.getReceiptId());
                if (null != receiptDO && receiptDO.getState().equals(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue())) {
                    ReceiptDO updateReceipt = new ReceiptDO();
                    updateReceipt.setId(receiptDO.getId());
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
                    receiptMapper.updateById(updateReceipt);

                    //更新收该收款单下的应收为收款中状态
                    LambdaUpdateWrapper<ReceivableDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
                    lambdaUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
                    lambdaUpdateWrapper.set(ReceivableDO::getState, 1);
                    receivableService.update(lambdaUpdateWrapper);
                }


            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceiptItem.setStatus(ReceiptItemStatusEnum.DRAFT.getValue());
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceiptItem.setStatus(ReceiptItemStatusEnum.DRAFT.getValue());
            }
            receiptItemMapper.updateById(updateReceiptItem);
        } else if (StrUtil.equals(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF_NO.getKey(), bmpKey)) {
            //收款单明细核销反审核
            if (null == receiptApprovalDO.getReceiptItemId()) {
                throw exception(RECEIPT_ITEM_NOT_FOUND);
            }
            ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(receiptApprovalDO.getReceiptItemId());
            if (null == receiptItemDO) {
                throw exception(RECEIPT_ITEM_NOT_FOUND);
            }
            ReceiptItemDO updateReceiptItem = new ReceiptItemDO();
            updateReceiptItem.setId(receiptItemDO.getId());
            updateReceiptItem.setBmpStatus(result);
            if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                updateReceiptItem.setStatus(ReceiptItemStatusEnum.DRAFT.getValue());
                receiptItemMapper.updateById(updateReceiptItem);
                //需要根据应收明细核销状态 变更成部分核销或者待核销
                ReceiptDO updateReceipt = new ReceiptDO();
                updateReceipt.setId(receiptItemDO.getReceiptId());
                LambdaQueryWrapper<ReceiptItemDO> receiptItemQueryWrapper = new LambdaQueryWrapper();
                receiptItemQueryWrapper.eq(ReceiptItemDO::getReceiptId, receiptItemDO.getReceiptId());
                receiptItemQueryWrapper.eq(ReceiptItemDO::getStatus, ReceiptItemStatusEnum.WRITE_OFF.getValue());
                if (receiptItemMapper.selectCount(receiptItemQueryWrapper) == 0) {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_WAITING.getValue());
                } else {
                    updateReceipt.setState(ReceiptStatusEnum.WRITE_OFF_PART_ING.getValue());
                }
                receiptMapper.updateById(updateReceipt);
            } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                updateReceiptItem.setStatus(ReceiptItemStatusEnum.WRITE_OFF.getValue());
                receiptItemMapper.updateById(updateReceiptItem);
            } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                updateReceiptItem.setStatus(ReceiptItemStatusEnum.WRITE_OFF.getValue());
                receiptItemMapper.updateById(updateReceiptItem);
            }

        }
        ReceiptApprovalDO updateReceiptApproval = new ReceiptApprovalDO();
        updateReceiptApproval.setId(receiptApprovalDO.getId());
        updateReceiptApproval.setStatus(result);
        receiptApprovalService.updateById(updateReceiptApproval);
    }
}
