package cn.iocoder.yudao.module.wealth.service.commissionPayable;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.wealth.vo.commissionPayable.*;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayable.CommissionPayableDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 订单佣金应付款 Service 接口
 *
 * @author 郑屹
 */
public interface CommissionPayableService extends IService<CommissionPayableDO> {

    /**
     * 创建订单佣金应付款
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCommissionPayable(@Valid CommissionPayableCreateReqVO createReqVO);

    /**
     * 更新订单佣金应付款
     * @param updateReqVO 更新信息
     */
    void updateCommissionPayable(@Valid CommissionPayableUpdateReqVO updateReqVO);

    /**
     * 删除订单佣金应付款
     * @param id 编号
     */
    void deleteCommissionPayable(Long id);

    /**
     * 获得订单佣金应付款
     * @param id 编号
     * @return 订单佣金应付款
     */
    CommissionPayableDO getCommissionPayable(Long id);

    /**
     * 获得订单佣金应付款列表
     * @param ids 编号
     * @return 订单佣金应付款列表
     */
    List<CommissionPayableDO> getCommissionPayableList(Collection<Long> ids);

    /**
     * 获得订单佣金应付款分页
     * @param page 分页查询
     * @param query 查询
     * @return 订单佣金应付款分页
     */
    PageResult<CommissionPayableBackVO> getCommissionPayablePage(CommissionPayableQueryVO query, PageVO page);

    /**
     * 获得订单佣金应付款列表, 用于 Excel 导出
     * @param query 查询
     * @return 订单佣金应付款列表
     */
    List<CommissionPayableBackVO> getCommissionPayableList(CommissionPayableQueryVO query);
}
