package cn.iocoder.yudao.module.wealth.service.commissionPayable;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.wealth.convert.payable.PayableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.wealth.vo.commissionPayable.*;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayable.CommissionPayableDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.wealth.convert.commissionPayable.CommissionPayableConvert;
import cn.iocoder.yudao.module.wealth.dal.mysql.commissionPayable.CommissionPayableMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 订单佣金应付款 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class CommissionPayableServiceImpl extends AbstractService<CommissionPayableMapper, CommissionPayableDO> implements CommissionPayableService {

    @Resource
    private CommissionPayableMapper commissionPayableMapper;

    @Override
    public Long createCommissionPayable(CommissionPayableCreateReqVO createReqVO) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        // TODO 具体业务待完善

        // 插入
        CommissionPayableDO commissionPayable = CommissionPayableConvert.INSTANCE.convert(createReqVO);
        commissionPayable.setCreateTime(new Date());
        commissionPayable.setCreator(creator);
        commissionPayable.setUpdateTime(new Date());
        commissionPayable.setUpdater(creator);
        commissionPayableMapper.insert(commissionPayable);
        // 返回
        return commissionPayable.getId();
    }

    @Override
    public void updateCommissionPayable(CommissionPayableUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCommissionPayableExists(updateReqVO.getId());
        // 更新
        CommissionPayableDO updateObj = CommissionPayableConvert.INSTANCE.convert(updateReqVO);
        commissionPayableMapper.updateById(updateObj);
    }

    @Override
    public void deleteCommissionPayable(Long id) {
        // 校验存在
        this.validateCommissionPayableExists(id);
        // 删除
        commissionPayableMapper.deleteById(id);
    }

    private void validateCommissionPayableExists(Long id) {
        if (commissionPayableMapper.selectById(id) == null) {
            throw exception(COMMISSION_PAYABLE_NOT_EXISTS);
        }
    }

    @Override
    public CommissionPayableDO getCommissionPayable(Long id) {
        return commissionPayableMapper.selectById(id);
    }

    @Override
    public List<CommissionPayableDO> getCommissionPayableList(Collection<Long> ids) {
        return commissionPayableMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CommissionPayableBackVO> getCommissionPayablePage(CommissionPayableQueryVO query, PageVO page) {
        page.setField("cp.id");
        IPage<CommissionPayableBackVO> mpPage = MyBatisUtils.buildPage(page);
        commissionPayableMapper.getCommissionPayablePage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public List<CommissionPayableBackVO> getCommissionPayableList(CommissionPayableQueryVO query) {
        return commissionPayableMapper.getCommissionPayableList(query);
//        return commissionPayableMapper.selectList(query);
    }
}
