package cn.iocoder.yudao.module.wealth.service.commissionPayment;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.wealth.vo.commissionPayment.*;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayment.CommissionPaymentDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.wealth.vo.payment.FinancePaymentCancelVo;
import cn.iocoder.yudao.module.wealth.vo.payment.FinancePaymentVo;

/**
 * 佣金付款单 Service 接口
 *
 * @author 郑屹
 */
public interface CommissionPaymentService extends IService<CommissionPaymentDO> {

    /**
     * 创建佣金付款单
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCommissionPayment(@Valid CommissionPaymentCreateReqVO createReqVO);

    /**
     * 更新佣金付款单
     * @param updateReqVO 更新信息
     */
    void updateCommissionPayment(@Valid CommissionPaymentUpdateReqVO updateReqVO);

    /**
     * 删除佣金付款单
     * @param id 编号
     */
    void deleteCommissionPayment(Long id);

    /**
     * 获得佣金付款单
     * @param id 编号
     * @return 佣金付款单
     */
    CommissionPaymentDO getCommissionPayment(Long id);

    /**
     * 获得佣金付款单列表
     * @param ids 编号
     * @return 佣金付款单列表
     */
    List<CommissionPaymentDO> getCommissionPaymentList(Collection<Long> ids);

    /**
     * 获得佣金付款单分页
     * @param page 分页查询
     * @param query 查询
     * @return 佣金付款单分页
     */
    PageResult<CommissionPaymentBackVO> getCommissionPaymentPage(CommissionPaymentQueryVO query, PageVO page);

    /**
     * 获得佣金付款单列表
     * @param query 查询
     * @return 佣金付款单列表
     */
    List<CommissionPaymentBackVO> getCommissionPaymentList(CommissionPaymentQueryVO query);

    /**
     * 反审
     *
     * @param financePaymentVo
     */
    void verifyPaymentCancel(FinancePaymentVo financePaymentVo);

    /**
     * 核销
     *
     * @param financePaymentVo
     */
    void verification(FinancePaymentVo financePaymentVo);

    /**
     * 反核销
     *
     * @param financePaymentVo
     */
    void verificationCancel(FinancePaymentVo financePaymentVo);

    void cancelFinancePaymentWriteOffNo(FinancePaymentCancelVo createReqVO);

    void cancelFinancePaymentWriteOff(FinancePaymentCancelVo createReqVO);

    void cancelFinancePaymentApprovalNo(FinancePaymentCancelVo createReqVO);

    void cancelFinancePaymentApproval(FinancePaymentCancelVo createReqVO);

    CommissionPaymentBackVO info(Long id);
}
