package cn.iocoder.yudao.module.wealth.service.commissionPayment;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.apollo.core.event.OrderNumberLogEvent;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.wealth.convert.payment.PaymentConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayable.CommissionPayableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payment.PaymentDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.paymentApproval.PaymentApprovalDO;
import cn.iocoder.yudao.module.wealth.enums.PaymentStatusEnum;
import cn.iocoder.yudao.module.wealth.service.commissionPayable.CommissionPayableService;
import cn.iocoder.yudao.module.wealth.service.paymentApproval.PaymentApprovalService;
import cn.iocoder.yudao.module.wealth.vo.commissionPayable.CommissionPayableBackVO;
import cn.iocoder.yudao.module.wealth.vo.commissionPayable.CommissionPayableQueryVO;
import cn.iocoder.yudao.module.wealth.vo.commissionPayable.CommissionPayableReqVO;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableReqVO;
import cn.iocoder.yudao.module.wealth.vo.payment.FinancePaymentCancelVo;
import cn.iocoder.yudao.module.wealth.vo.payment.FinancePaymentVo;
import cn.iocoder.yudao.module.wealth.vo.payment.PaymentVerifyVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.wealth.vo.commissionPayment.*;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayment.CommissionPaymentDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.wealth.convert.commissionPayment.CommissionPaymentConvert;
import cn.iocoder.yudao.module.wealth.dal.mysql.commissionPayment.CommissionPaymentMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.COMMISSION_PAYMENT_NOT_UPDATE;

/**
 * 佣金付款单 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class CommissionPaymentServiceImpl extends AbstractService<CommissionPaymentMapper, CommissionPaymentDO> implements CommissionPaymentService {

    @Resource
    private CommissionPaymentMapper commissionPaymentMapper;
    @Resource
    private CurrencyService currencyService;
    @Resource
    private CommissionPayableService commissionPayableService;
    @Autowired
    private ApplicationContext applicationContext;
    @Resource
    PaymentApprovalService paymentApprovalService;
    @Resource
    BpmCreateServiceFactory bpmCreateServiceFactory;

    private static final String COMMISSION_PAY_NEW_NUMBER = "commission:payment:new:number";


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createCommissionPayment(CommissionPaymentCreateReqVO createReqVO) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        // 插入
        CommissionPaymentDO commissionPayment = CommissionPaymentConvert.INSTANCE.convert(createReqVO);

        OrderNumberLogEvent event = new OrderNumberLogEvent();
        event.setType(5);
        event.setNumberKey(COMMISSION_PAY_NEW_NUMBER);
        applicationContext.publishEvent(event);
        String newPaymentNo = CodeUtils.getOddCommissionPayNumbers(event.getNumber());
        commissionPayment.setPaymentNo(newPaymentNo);

        event.setNewNumber(newPaymentNo);
        applicationContext.publishEvent(event);

        commissionPayment.setPaymentAddBy(null == user ? "" : user.getUsername());
        commissionPayment.setPaymentAddId(null == user ? null : user.getId());

        commissionPayment.setCreateTime(new Date());
        commissionPayment.setCreator(creator);
        commissionPayment.setUpdateTime(new Date());
        commissionPayment.setUpdater(creator);


        if (StringUtils.equals(createReqVO.getAddType(), "0")) {
            commissionPayment.setState(PaymentStatusEnum.DRAFT.getValue());
        }


        commissionPaymentMapper.insert(commissionPayment);

        if (StringUtils.equals(createReqVO.getAddType(), "1")) {
            validatePaymentBmpStatus(commissionPayment.getId());
            //创建收款单并发起审核流程
            PaymentApprovalDO approvalDO = new PaymentApprovalDO();
            approvalDO.setPaymentId(commissionPayment.getId());
            approvalDO.setPaymentNo(commissionPayment.getPaymentNo());
            approvalDO.setBmpKey(WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_APPROVE.getKey());
            approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            paymentApprovalService.save(approvalDO);
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_APPROVE.getKey(), commissionPayment.getPaymentNo(), createReqVO.getCopyUserList());
            approvalDO.setBmpId(bpmId);
            paymentApprovalService.updateById(approvalDO);

            CommissionPaymentDO update = new CommissionPaymentDO();
            update.setId(commissionPayment.getId());
            update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            update.setBmpId(bpmId);
            update.setState(PaymentStatusEnum.APPROVE_ING.getValue());
            commissionPaymentMapper.updateById(update);
        }


        //修改应付明细里面的付款单信息
        List<CommissionPayableReqVO> payableReqVOList = createReqVO.getCommissionPayableReqVOList();
        if (Objects.isNull(payableReqVOList) || CollectionUtil.isEmpty(payableReqVOList)){
            throw exception(COMMISSION_PAYABLE_NOT_NULL);
        }
        for (CommissionPayableReqVO payableReqVO : payableReqVOList) {
            CommissionPayableDO payableDO = commissionPayableService.getCommissionPayable(payableReqVO.getId());
            if (Objects.isNull(payableDO)){
                throw exception(COMMISSION_PAYABLE_NOT_EXISTS);
            }
            if (Objects.nonNull(payableDO.getPaymentId()) && (payableDO.getState() == 1 || payableDO.getState() == 2)){
                throw exception(COMMISSION_PAYABLE_REQUEST_PAYMENT);
            }
            //回写应付单信息到应付信息中
            payableDO.setId(payableReqVO.getId());
            payableDO.setPaymentId(commissionPayment.getId());
            payableDO.setPaymentNo(commissionPayment.getPaymentNo());
            payableDO.setInvoiceNumber(payableReqVO.getInvoiceNumber());
            payableDO.setTotal(payableReqVO.getTotal());
            payableDO.setTaxRate(payableReqVO.getTaxRate());
            payableDO.setExchangeRate(payableReqVO.getExchangeRate());
            payableDO.setState(1);
            commissionPayableService.updateById(payableDO);
        }


        // 返回
        return commissionPayment.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateCommissionPayment(CommissionPaymentUpdateReqVO updateReqVO) {
        // 校验存在
        CommissionPaymentDO commissionPaymentDO = commissionPaymentMapper.selectById(updateReqVO.getId());
        if (commissionPaymentDO == null) {
            throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
        }

        //判断付款单状态能不能进行更改
        if (!Objects.equals(commissionPaymentDO.getState(), PaymentStatusEnum.DRAFT.getValue()) && !Objects.equals(commissionPaymentDO.getState(), PaymentStatusEnum.APPROVE_FAIL.getValue())) {
            throw exception(COMMISSION_PAYMENT_NOT_UPDATE);
        }

        // 更新
        CommissionPaymentDO updateObj = CommissionPaymentConvert.INSTANCE.convert(updateReqVO);
        if (StringUtils.equals(updateReqVO.getAddType(), "0")) {
            updateReqVO.setState(PaymentStatusEnum.DRAFT.getValue());
        }

        if (StringUtils.equals(updateReqVO.getAddType(), "1")) {
            validatePaymentBmpStatus(commissionPaymentDO.getId());
            //创建收款单并发起审核流程
            PaymentApprovalDO approvalDO = new PaymentApprovalDO();
            approvalDO.setPaymentId(commissionPaymentDO.getId());
            approvalDO.setPaymentNo(commissionPaymentDO.getPaymentNo());
            approvalDO.setBmpKey(WorkFlowEmus.FINANCE_PAYMENT_APPROVE.getKey());
            approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            paymentApprovalService.save(approvalDO);
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_APPROVE.getKey(), commissionPaymentDO.getPaymentNo(), updateReqVO.getCopyUserList());
            approvalDO.setBmpId(bpmId);
            paymentApprovalService.updateById(approvalDO);

            updateObj.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            updateObj.setState(PaymentStatusEnum.APPROVE_ING.getValue());
            updateObj.setBmpId(bpmId);
        }

        commissionPaymentMapper.updateById(updateObj);
        //修改应付明细里面的付款单信息
        List<CommissionPayableDO> oldCommissionPayableDOList = commissionPayableService.list(new LambdaQueryWrapper<CommissionPayableDO>()
                .eq(CommissionPayableDO::getPaymentId, updateReqVO.getId()));
        List<CommissionPayableReqVO> commissionPayableReqVOList = updateReqVO.getCommissionPayableReqVOList();
        if (Objects.isNull(commissionPayableReqVOList) || CollectionUtil.isEmpty(commissionPayableReqVOList)){
            throw exception(COMMISSION_PAYABLE_NOT_NULL);
        }
        for (CommissionPayableReqVO commissionPayableReqVO : commissionPayableReqVOList) {
            Boolean flag = Boolean.TRUE;
            if (CollectionUtil.isNotEmpty(oldCommissionPayableDOList)){
                for (CommissionPayableDO payableDO : oldCommissionPayableDOList) {
                    payableDO.setDel(Boolean.TRUE);
                    if (Objects.equals(commissionPayableReqVO.getId(), payableDO.getId())){
                        payableDO.setDel(Boolean.FALSE);
                        flag = Boolean.FALSE;
                        payableDO.setInvoiceNumber(commissionPayableReqVO.getInvoiceNumber());
                        payableDO.setTotal(commissionPayableReqVO.getTotal());
                        payableDO.setTaxRate(commissionPayableReqVO.getTaxRate());
                        payableDO.setExchangeRate(commissionPayableReqVO.getExchangeRate());
                        payableDO.setState(1);
                        commissionPayableService.updateById(payableDO);
                    }
                }
            }
            if (flag) {
                // 新增的请款应付，回写付款单信息到应付信息中
                CommissionPayableDO payableDO = commissionPayableService.getCommissionPayable(commissionPayableReqVO.getId());
                if (Objects.isNull(payableDO)) {
                    throw exception(COMMISSION_PAYABLE_NOT_EXISTS);
                }
                if (Objects.nonNull(payableDO.getPaymentId()) && (payableDO.getState() == 1 || payableDO.getState() == 2)) {
                    throw exception(COMMISSION_PAYABLE_REQUEST_PAYMENT);
                }
                payableDO.setId(commissionPayableReqVO.getId());
                payableDO.setPaymentId(updateObj.getId());
                payableDO.setPaymentNo(updateObj.getPaymentNo());
                payableDO.setInvoiceNumber(commissionPayableReqVO.getInvoiceNumber());
                payableDO.setTotal(commissionPayableReqVO.getTotal());
                payableDO.setTaxRate(commissionPayableReqVO.getTaxRate());
                payableDO.setExchangeRate(commissionPayableReqVO.getExchangeRate());
                payableDO.setState(1);
                commissionPayableService.updateById(payableDO);
            }
        }

        for (CommissionPayableDO payableDO : oldCommissionPayableDOList) {
            if (payableDO.getDel()) {
                // 将被剔除的应付删除请款id及编号，状态置为待付款
                commissionPayableService.update(new LambdaUpdateWrapper<CommissionPayableDO>()
                        .set(CommissionPayableDO::getPaymentId, null)
                        .set(CommissionPayableDO::getPaymentNo, null)
                        .set(CommissionPayableDO::getState, 0)
                        .eq(CommissionPayableDO::getId, payableDO.getId()));
            }
        }
    }

    @Override
    public void deleteCommissionPayment(Long id) {
        // 校验存在
        this.validateCommissionPaymentExists(id);
        // 删除
        commissionPaymentMapper.deleteById(id);
        // 删除付款单后，同步更新应付佣金明细的状态
        commissionPayableService.update(new LambdaUpdateWrapper<CommissionPayableDO>()
                .set(CommissionPayableDO::getPaymentId, null)
                .set(CommissionPayableDO::getPaymentNo, null)
                .set(CommissionPayableDO::getState, 0)
                .eq(CommissionPayableDO::getPaymentId, id));
    }

    private void validateCommissionPaymentExists(Long id) {
        if (commissionPaymentMapper.selectById(id) == null) {
            throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
        }
    }

    @Override
    public CommissionPaymentDO getCommissionPayment(Long id) {
        return commissionPaymentMapper.selectById(id);
    }

    @Override
    public List<CommissionPaymentDO> getCommissionPaymentList(Collection<Long> ids) {
        return commissionPaymentMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CommissionPaymentBackVO> getCommissionPaymentPage(CommissionPaymentQueryVO query, PageVO page) {
        page.setField("cp.id");
        IPage<CommissionPaymentBackVO> mpPage = MyBatisUtils.buildPage(page);
        commissionPaymentMapper.getCommissionPaymentPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public List<CommissionPaymentBackVO> getCommissionPaymentList(CommissionPaymentQueryVO query) {
        return commissionPaymentMapper.getCommissionPaymentList(query);
    }

    @Override
    public void verifyPaymentCancel(FinancePaymentVo financePaymentVo) {
        validatePaymentBmpStatus(financePaymentVo.getPaymentId());

        //创建付款单反审核流程
        PaymentApprovalDO approvalDO = new PaymentApprovalDO();
        approvalDO.setPaymentId(financePaymentVo.getPaymentId());
        approvalDO.setPaymentNo(financePaymentVo.getPaymentNo());
        approvalDO.setReason(financePaymentVo.getReason());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_APPROVE_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        paymentApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_APPROVE_NO.getKey(), financePaymentVo.getPaymentNo(), financePaymentVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        paymentApprovalService.updateById(approvalDO);

        CommissionPaymentDO update = new CommissionPaymentDO();
        update.setId(financePaymentVo.getPaymentId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(PaymentStatusEnum.APPROVE_NEGATION.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);


       /* CommissionPaymentDO paymentDO = this.getById(id);
        if (paymentDO == null) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        if (paymentDO.getState() == 2) {
            LoginUser user = SecurityFrameworkUtils.getLoginUser();
            String creator = String.valueOf(null == user ? "" : user.getId());

            CommissionPaymentDO updatePaymentDO = new CommissionPaymentDO();
            updatePaymentDO.setId(id);
            updatePaymentDO.setState(1);

            updatePaymentDO.setUpdateTime(new Date());
            updatePaymentDO.setUpdater(creator);

            this.updateById(updatePaymentDO);

        } else {
            throw exception(PAYMENT_VERIFY_STATUS_ERROR);
        }*/
    }

    @Override
    public void verification(FinancePaymentVo financePaymentVo) {
        validatePaymentBmpStatus(financePaymentVo.getPaymentId());

        //创建付款单核销流程
        PaymentApprovalDO approvalDO = new PaymentApprovalDO();
        approvalDO.setPaymentId(financePaymentVo.getPaymentId());
        approvalDO.setPaymentNo(financePaymentVo.getPaymentNo());
        approvalDO.setReason(financePaymentVo.getReason());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_WRITE_OFF.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        paymentApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_WRITE_OFF.getKey(), financePaymentVo.getPaymentNo(), financePaymentVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        paymentApprovalService.updateById(approvalDO);

        CommissionPaymentDO update = new CommissionPaymentDO();
        update.setId(financePaymentVo.getPaymentId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(PaymentStatusEnum.WRITE_OFF_APPROVE_ING.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);

        /*PaymentDO paymentDO = this.getById(id);
        if (paymentDO == null) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        if (paymentDO.getState() == 2) {
            LoginUser user = SecurityFrameworkUtils.getLoginUser();
            PaymentDO updatePaymentDO = new PaymentDO();
            updatePaymentDO.setId(id);
            updatePaymentDO.setState(4);
            updatePaymentDO.setWriteOffId(user.getId());
            updatePaymentDO.setWriteOffName(user.getUsername());
            updatePaymentDO.setWriteOffAt(new Date());
            this.updateById(updatePaymentDO);
        } else {
            throw exception(PAYMENT_VERIFICATION);
        }*/
    }

    @Override
    public void verificationCancel(FinancePaymentVo financePaymentVo) {
        validatePaymentBmpStatus(financePaymentVo.getPaymentId());
        //创建付款单反核销流程
        PaymentApprovalDO approvalDO = new PaymentApprovalDO();
        approvalDO.setPaymentId(financePaymentVo.getPaymentId());
        approvalDO.setPaymentNo(financePaymentVo.getPaymentNo());
        approvalDO.setReason(financePaymentVo.getReason());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_WRITE_OFF_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        paymentApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_COMMISSION_PAYMENT_WRITE_OFF_NO.getKey(), financePaymentVo.getPaymentNo(), financePaymentVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        paymentApprovalService.updateById(approvalDO);

        CommissionPaymentDO update = new CommissionPaymentDO();
        update.setId(financePaymentVo.getPaymentId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setState(PaymentStatusEnum.WRITE_OFF_NO_APPROVE_ING.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);


        /*if (paymentDO.getState() == 4) {
            LoginUser user = SecurityFrameworkUtils.getLoginUser();
            String creator = String.valueOf(null == user ? "" : user.getId());

            PaymentDO updatePaymentDO = new PaymentDO();
            updatePaymentDO.setId(id);
            updatePaymentDO.setState(1);

            updatePaymentDO.setUpdateTime(new Date());
            updatePaymentDO.setUpdater(creator);

            this.updateById(updatePaymentDO);

        } else {
            throw exception(PAYMENT_VERIFICATION_ERROR);
        }*/

    }



    @Override
    public void cancelFinancePaymentWriteOffNo(FinancePaymentCancelVo createReqVO) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        CommissionPaymentDO paymentDO = validatePaymentBmpCancel(createReqVO.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), createReqVO.getRemark());
    }

    @Override
    public void cancelFinancePaymentWriteOff(FinancePaymentCancelVo createReqVO) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        CommissionPaymentDO paymentDO = validatePaymentBmpCancel(createReqVO.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), createReqVO.getRemark());
    }

    @Override
    public void cancelFinancePaymentApprovalNo(FinancePaymentCancelVo createReqVO) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        CommissionPaymentDO paymentDO = validatePaymentBmpCancel(createReqVO.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), createReqVO.getRemark());
    }

    @Override
    public void cancelFinancePaymentApproval(FinancePaymentCancelVo createReqVO) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        CommissionPaymentDO paymentDO = validatePaymentBmpCancel(createReqVO.getPaymentId());
        bpmCreateServiceFactory.cancelBmpById(userId, paymentDO.getBmpId(), createReqVO.getRemark());
    }

    @Override
    public CommissionPaymentBackVO info(Long id) {
        CommissionPaymentBackVO backVO = commissionPaymentMapper.info(id);
        if (Objects.isNull(backVO)){
            throw exception(COMMISSION_PAYMENT_NOT_EXISTS);
        }
        CommissionPayableQueryVO queryVO = new CommissionPayableQueryVO();
        queryVO.setPaymentId(id);
        List<CommissionPayableBackVO> commissionPayableList = commissionPayableService.getCommissionPayableList(queryVO);
        if (CollectionUtil.isNotEmpty(commissionPayableList)){
            backVO.setPayableBackVOList(commissionPayableList);
//            // 根据汇率计算当前应付总rmb
//            BigDecimal totalAmount = commissionPayableList.stream()
//                    .filter(p -> StringUtils.isNotBlank(p.getExchangeRate()) && new BigDecimal(p.getExchangeRate()).compareTo(BigDecimal.ZERO) > 0)
//                    .map(payable -> new BigDecimal(payable.getExchangeRate()).multiply(payable.getTotalAmount()))
//                    .reduce(BigDecimal.ZERO,BigDecimal::add);
            BigDecimal totalAmount = commissionPayableList.stream().map(CommissionPayableBackVO::getTotal).reduce(BigDecimal.ZERO,BigDecimal::add);
            backVO.setTotalAmount(totalAmount.setScale(2, RoundingMode.HALF_UP));
        }
        return backVO;
    }

    private void validatePaymentBmpStatus(Long id) {
        CommissionPaymentDO paymentDO = commissionPaymentMapper.selectById(id);
        if (null == paymentDO) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        if (null != paymentDO.getBmpStatus() && Objects.equals(paymentDO.getBmpStatus(), BpmProcessInstanceResultEnum.PROCESS.getResult())) {
            throw exception(PAYMENT_APPROVING);
        }
    }

    private CommissionPaymentDO validatePaymentBmpCancel(Long id) {
        CommissionPaymentDO paymentDO = this.getCommissionPayment(id);
        if (null == paymentDO) {
            throw exception(PAYMENT_NOT_EXISTS);
        }
        if (null == paymentDO.getBmpStatus()) {

            throw exception(BMP_NOT);
        }

        if (!Objects.equals(paymentDO.getBmpStatus(), BpmProcessInstanceResultEnum.PROCESS.getResult())) {
            throw exception(BMP_CANCEL);
        }

        return paymentDO;
    }
}
