package cn.iocoder.yudao.module.wealth.service.payable;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.wealth.vo.payable.*;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 应付款 Service 接口
 *
 * @author 郑屹
 */
public interface PayableService extends IService<PayableDO> {

    /**
     * 创建应付款
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPayable(@Valid PayableCreateReqVO createReqVO);

    /**
     * 更新应付款
     * @param updateReqVO 更新信息
     */
    void updatePayable(@Valid PayableUpdateReqVO updateReqVO);

    /**
     * 删除应付款
     * @param id 编号
     */
    void deletePayable(Long id);

    /**
     * 获得应付款
     * @param id 编号
     * @return 应付款
     */
    PayableDO getPayable(Long id);

    /**
     * 获得应付款列表
     * @param ids 编号
     * @return 应付款列表
     */
    List<PayableDO> getPayableList(Collection<Long> ids);

    /**
     * 获得应付款分页
     * @param page 分页查询
     * @param query 查询
     * @return 应付款分页
     */
    PageResult<PayableOrderBackVo> getPayablePage(PayableQueryVO query, PageVO page);

    /**
     * 获得应付款列表, 用于 Excel 导出
     * @param query 查询
     * @return 应付款列表
     */
    List<PayableDO> getPayableList(PayableQueryVO query);
}
