package cn.iocoder.yudao.module.wealth.service.payable;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.wealth.convert.payable.PayableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.payable.PayableMapper;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableOrderBackVo;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableQueryVO;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.PAYABLE_NOT_EXISTS;

/**
 * 应付款 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class PayableServiceImpl extends AbstractService<PayableMapper, PayableDO> implements PayableService {

    @Autowired
    private PayableMapper payableMapper;

    /*

     */
    @Autowired
    private AdminUserService adminUserService;

    @Override
    public Long createPayable(PayableCreateReqVO createReqVO) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        // 插入
        PayableDO payable = PayableConvert.INSTANCE.convert(createReqVO);
        payable.setCreateTime(new Date());
        payable.setCreator(creator);
        payable.setUpdateTime(new Date());
        payable.setUpdater(creator);
        payableMapper.insert(payable);
        // 返回
        return payable.getId();
    }

    @Override
    public void updatePayable(PayableUpdateReqVO updateReqVO) {
        // 校验存在
        this.validatePayableExists(updateReqVO.getId());
        // 更新
        PayableDO updateObj = PayableConvert.INSTANCE.convert(updateReqVO);
        payableMapper.updateById(updateObj);
    }

    @Override
    public void deletePayable(Long id) {
        // 校验存在
        this.validatePayableExists(id);
        // 删除
        payableMapper.deleteById(id);
    }

    private void validatePayableExists(Long id) {
        if (payableMapper.selectById(id) == null) {
            throw exception(PAYABLE_NOT_EXISTS);
        }
    }

    @Override
    public PayableDO getPayable(Long id) {
        return payableMapper.selectById(id);
    }

    @Override
    public List<PayableDO> getPayableList(Collection<Long> ids) {
        return payableMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<PayableOrderBackVo> getPayablePage(PayableQueryVO query,
                                                         PageVO page) {

        List<AdminUserDO> ListUser = adminUserService.getUsersList();
        Map<Long, String> userMap =
                ListUser.stream().collect(
                        Collectors.toMap((AdminUserDO::getId),
                                AdminUserDO::getNickname));

        IPage<PayableOrderBackVo> mpPage = MyBatisUtils.buildPage(page);
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("a.deleted", "0");
        queryWrapper.apply("a.payment_id is  null");
        queryWrapper.like(StringUtils.isNotBlank(query.getPayableNo()), "a.payable_no", query.getPayableNo());
        queryWrapper.like(StringUtils.isNotBlank(query.getContainerNo()), "a.container_no", query.getContainerNo());
        queryWrapper.like(StringUtils.isNotBlank(query.getSupplierName()), "a.supplier_name", query.getSupplierName());
        queryWrapper.eq(null != query.getFeeType(), "a.fee_type", query.getFeeType());
        queryWrapper.eq(StringUtils.isNotBlank(query.getTransportId()), "a.transport_id", query.getTransportId());
        queryWrapper.eq(StringUtils.isNotBlank(query.getDepartureId()), "a.start_warehouse_id", query.getDepartureId());
        queryWrapper.eq(StringUtils.isNotBlank(query.getObjectiveId()), "a.dest_warehouse_id", query.getObjectiveId());

        //添加的查询条件
        if (query.getBeginCreateTime() != null &&
                query.getEndCreateTime() != null) {
            if (query.getDateType() == 1) {
                //预装时间,
                queryWrapper.between("b.yz_date",
                        query.getBeginCreateTime(), query.getEndCreateTime());

            } else if (query.getDateType() == 2) {
                //装柜时间,
                queryWrapper.between("b.zg_date",
                        query.getBeginCreateTime(), query.getEndCreateTime());

            } else if (query.getDateType() == 3) {
                //起运时间
                queryWrapper.between("b.qy_date",
                        query.getBeginCreateTime(), query.getEndCreateTime());

            } else if (query.getDateType() == 4) {
                //到港时间
                queryWrapper.between("b.dg_date",
                        query.getBeginCreateTime(), query.getEndCreateTime());

            } else if (query.getDateType() == 5) {
                //清关时间
                queryWrapper.between("b.qg_date",
                        query.getBeginCreateTime(), query.getEndCreateTime());

            } else if (query.getDateType() == 6) {
                //卸柜时间
                queryWrapper.between("b.th_date",
                        query.getBeginCreateTime(), query.getEndCreateTime());

            } else if (query.getDateType() == 7) {
                //费用产生时间
                queryWrapper.between("a.freecsdate",
                        query.getBeginCreateTime(), query.getEndCreateTime());
            }
        }

        //queryWrapper.between()
        payableMapper.selectListPage(mpPage, queryWrapper);

        //
        for (PayableOrderBackVo v : mpPage.getRecords()
        ) {
            if (v.getTransportId() == 1) {
                v.setTransportTypeName("海运拼柜");
            } else if (v.getTransportId() == 3) {
                v.setTransportTypeName("专线空运");
            }

            String sCreator = "";
            if (v.getCreator() == null) {
                v.setCreator("");
            } else {
                long l = Long.parseLong(v.getCreator());
                v.setCreator(userMap.get(l));
            }

            if (v.getUpdater() == null) {
                v.setUpdater("");
            } else {
                long l = Long.parseLong(v.getUpdater());
                v.setUpdater(userMap.get(l));
            }
        }
        return PageResult.of(mpPage);
    }

    @Override
    public List<PayableDO> getPayableList(PayableQueryVO query) {
        return payableMapper.selectList(query);
    }
}
