package cn.iocoder.yudao.module.wealth.service.payment;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payment.PaymentDO;
import cn.iocoder.yudao.module.wealth.vo.payment.*;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptVo;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 付款单 Service 接口
 *
 * @author 郑屹
 */
public interface PaymentService extends IService<PaymentDO> {

    /**
     * 创建付款单
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPayment(@Valid PaymentCreateReqVO createReqVO);

    /**
     * 更新付款单
     *
     * @param updateReqVO 更新信息
     */
    void updatePayment(@Valid PaymentUpdateReqVO updateReqVO);

    /**
     * 删除付款单
     *
     * @param id 编号
     */
    void deletePayment(Long id);

    /**
     * 获得付款单
     *
     * @param id 编号
     * @return 付款单
     */
    PaymentDO getPayment(Long id);

    /**
     * 获得付款单列表
     *
     * @param ids 编号
     * @return 付款单列表
     */
    List<PaymentDO> getPaymentList(Collection<Long> ids);

    /**
     * 获得付款单分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 付款单分页
     */
    PageResult<PaymentDO> getPaymentPage(PaymentQueryVO query, PageVO page);

    /**
     * 获得付款单列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 付款单列表
     */
    List<PaymentDO> getPaymentList(PaymentQueryVO query);

    /**
     * 付款单审核
     */
    void verifyPayment(PaymentVerifyVo paymentVerifyVo);

    /**
     * 反审
     *
     * @param financePaymentVo
     */
    void verifyPaymentCancel(FinancePaymentVo financePaymentVo);

    /**
     * 核销
     *
     * @param financePaymentVo
     */
    void verification(FinancePaymentVo financePaymentVo);

    /**
     * 反核销
     *
     * @param financePaymentVo
     */
    void verificationCancel(FinancePaymentVo financePaymentVo);

    /**
     * 取消付款单审核
     *
     * @param financePaymentCancelVo
     */
    void cancelFinancePaymentApproval(FinancePaymentCancelVo financePaymentCancelVo);
    /**
     * 取消付款单反审核
     *
     * @param financePaymentCancelVo
     */
    void cancelFinancePaymentApprovalNo(FinancePaymentCancelVo financePaymentCancelVo);
    /**
     * 取消付款单核销
     *
     * @param financePaymentCancelVo
     */
    void cancelFinancePaymentWriteOff(FinancePaymentCancelVo financePaymentCancelVo);
    /**
     * 取消付款单反核销
     *
     * @param financePaymentCancelVo
     */
    void cancelFinancePaymentWriteOffNo(FinancePaymentCancelVo financePaymentCancelVo);
}
