package cn.iocoder.yudao.module.wealth.service.paymentApproval;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.convert.paymentApproval.PaymentApprovalConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.paymentApproval.PaymentApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.paymentApproval.PaymentApprovalMapper;
import cn.iocoder.yudao.module.wealth.vo.paymentApproval.PaymentApprovalCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.paymentApproval.PaymentApprovalQueryVO;
import cn.iocoder.yudao.module.wealth.vo.paymentApproval.PaymentApprovalUpdateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

/**
 * 付款单审核 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class PaymentApprovalServiceImpl extends AbstractService<PaymentApprovalMapper, PaymentApprovalDO> implements PaymentApprovalService {

    @Resource
    private PaymentApprovalMapper paymentApprovalMapper;

    @Override
    public Long createPaymentApproval(PaymentApprovalCreateReqVO createReqVO) {
        // 插入
        PaymentApprovalDO paymentApproval = PaymentApprovalConvert.INSTANCE.convert(createReqVO);
        paymentApprovalMapper.insert(paymentApproval);
        // 返回
        return paymentApproval.getId();
    }

    @Override
    public void updatePaymentApproval(PaymentApprovalUpdateReqVO updateReqVO) {
        // 校验存在
        this.validatePaymentApprovalExists(updateReqVO.getId());
        // 更新
        PaymentApprovalDO updateObj = PaymentApprovalConvert.INSTANCE.convert(updateReqVO);
        paymentApprovalMapper.updateById(updateObj);
    }

    @Override
    public void deletePaymentApproval(Long id) {
        // 校验存在
        this.validatePaymentApprovalExists(id);
        // 删除
        paymentApprovalMapper.deleteById(id);
    }

    private void validatePaymentApprovalExists(Long id) {
        if (paymentApprovalMapper.selectById(id) == null) {
            //throw exception(PAYMENT_APPROVAL_NOT_EXISTS);
        }
    }

    @Override
    public PaymentApprovalDO getPaymentApproval(Long id) {
        return paymentApprovalMapper.selectById(id);
    }

    @Override
    public List<PaymentApprovalDO> getPaymentApprovalList(Collection<Long> ids) {
        return paymentApprovalMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<PaymentApprovalDO> getPaymentApprovalPage(PaymentApprovalQueryVO query, PageVO page) {
        return paymentApprovalMapper.selectPage(page, query);
    }

    @Override
    public List<PaymentApprovalDO> getPaymentApprovalList(PaymentApprovalQueryVO query) {
        return paymentApprovalMapper.selectList(query);
    }
}
