package cn.iocoder.yudao.module.wealth.service.receipt;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.vo.receipt.*;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 收款单 Service 接口
 *
 * @author 郑屹
 */
public interface ReceiptService extends IService<ReceiptDO> {

    /**
     * 创建收款单
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createReceipt(@Valid ReceiptCreateReqVO createReqVO);

    /**
     * 更新收款单
     *
     * @param updateReqVO 更新信息
     */
    void updateReceipt(@Valid ReceiptUpdateReqVO updateReqVO);


    /**
     * 获取开票信息
     *
     * @param receiptId 付款单ID
     */
    ReceiptBackVO getInvoicingInfo(Long receiptId);

    /**
     * 更新发票信息
     *
     * @param invoicingUpdateVO 更新信息
     */
    void updateInvoicingInfo(@Valid InvoicingUpdateVO invoicingUpdateVO);

    /**
     * 删除收款单
     *
     * @param id 编号
     */
    void deleteReceipt(Long id);

    /**
     * 获得收款单
     *
     * @param id 编号
     * @return 收款单
     */
    ReceiptDO getReceipt(Long id);

    /**
     * 获得收款单列表
     *
     * @param ids 编号
     * @return 收款单列表
     */
    List<ReceiptDO> getReceiptList(Collection<Long> ids);

    /**
     * 获得收款单分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 收款单分页
     */
    PageResult<ReceiptBackVO> getReceiptPage(ReceiptQueryVO query, PageVO page);

    /**
     * 获得收款单列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 收款单列表
     */
    List<ReceiptDO> getReceiptList(ReceiptQueryVO query);

    /**
     * 核销
     *
     * @param id
     */
    void verification(Long id);

    /**
     * 反核销
     *
     * @param id
     */
    void verificationCancel(Long id);

    /**
     * 创建收款单反审核流程
     */
    void createFinanceApproveNo(FinanceReceiptVo financeReceiptVo);

    /**
     * 创建收款核销流程
     */
    void createFinanceReceiptWriteOff(FinanceReceiptVo financeReceiptVo);

    /**
     * 创建收款单反核销流程
     */
    void createFinanceReceiptWriteOffNo(FinanceReceiptVo financeReceiptVo);

    /**
     * 取消收款单审核
     */
    void cancelFinanceReceiptApproval(FinanceReceiptCancelVo financeReceiptCancelVo);

    /**
     * 取消收款单反审核
     */
    void cancelFinanceReceiptApprovalNo(FinanceReceiptCancelVo financeReceiptCancelVo);

    /**
     * 取消收款单核销
     */
    void cancelFinanceReceiptWriteOff(FinanceReceiptCancelVo financeReceiptCancelVo);

    /**
     * 取消收款单反核销
     */
    void cancelFinanceReceiptWriteOffNo(FinanceReceiptCancelVo financeReceiptCancelVo);


    ReceiptExportDto getReceiptExportByOrderId(Long orderId);


}
