package cn.iocoder.yudao.module.wealth.service.receiptAccount;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountQueryVO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 收款单收款账户 Service 接口
 *
 * @author 系统管理员
 */
public interface ReceiptAccountService extends IService<ReceiptAccountDO> {

    /**
     * 创建收款单收款账户
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createReceiptAccount(@Valid ReceiptAccountCreateReqVO createReqVO);

    /**
     * 更新收款单收款账户
     *
     * @param updateReqVO 更新信息
     */
    void updateReceiptAccount(@Valid ReceiptAccountUpdateReqVO updateReqVO);

    /**
     * 删除收款单收款账户
     *
     * @param id 编号
     */
    void deleteReceiptAccount(Long id);

    /**
     * 获得收款单收款账户
     *
     * @param id 编号
     * @return 收款单收款账户
     */
    ReceiptAccountDO getReceiptAccount(Long id);

    /**
     * 获得收款单收款账户列表
     *
     * @param ids 编号
     * @return 收款单收款账户列表
     */
    List<ReceiptAccountDO> getReceiptAccountList(Collection<Long> ids);

    /**
     * 获得收款单收款账户分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 收款单收款账户分页
     */
    PageResult<ReceiptAccountDO> getReceiptAccountPage(ReceiptAccountQueryVO query, PageVO page);

    /**
     * 获得收款单收款账户列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 收款单收款账户列表
     */
    List<ReceiptAccountDO> getReceiptAccountList(ReceiptAccountQueryVO query);
}
