package cn.iocoder.yudao.module.wealth.service.receiptAccount;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.convert.receiptAccount.ReceiptAccountConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptAccount.ReceiptAccountMapper;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountQueryVO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountUpdateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIPT_ACCOUNT_NOT_EXISTS;

/**
 * 收款单收款账户 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ReceiptAccountServiceImpl extends AbstractService<ReceiptAccountMapper, ReceiptAccountDO> implements ReceiptAccountService {

    @Resource
    private ReceiptAccountMapper receiptAccountMapper;

    @Override
    public Long createReceiptAccount(ReceiptAccountCreateReqVO createReqVO) {
        // 插入
        ReceiptAccountDO receiptAccount = ReceiptAccountConvert.INSTANCE.convert(createReqVO);
        receiptAccountMapper.insert(receiptAccount);
        // 返回
        return receiptAccount.getId();
    }

    @Override
    public void updateReceiptAccount(ReceiptAccountUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateReceiptAccountExists(updateReqVO.getId());
        // 更新
        ReceiptAccountDO updateObj = ReceiptAccountConvert.INSTANCE.convert(updateReqVO);
        receiptAccountMapper.updateById(updateObj);
    }

    @Override
    public void deleteReceiptAccount(Long id) {
        // 校验存在
        this.validateReceiptAccountExists(id);
        // 删除
        receiptAccountMapper.deleteById(id);
    }

    private void validateReceiptAccountExists(Long id) {
        if (receiptAccountMapper.selectById(id) == null) {
            throw exception(RECEIPT_ACCOUNT_NOT_EXISTS);
        }
    }

    @Override
    public ReceiptAccountDO getReceiptAccount(Long id) {
        return receiptAccountMapper.selectById(id);
    }

    @Override
    public List<ReceiptAccountDO> getReceiptAccountList(Collection<Long> ids) {
        return receiptAccountMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceiptAccountDO> getReceiptAccountPage(ReceiptAccountQueryVO query, PageVO page) {
        return receiptAccountMapper.selectPage(page, query);
    }

    @Override
    public List<ReceiptAccountDO> getReceiptAccountList(ReceiptAccountQueryVO query) {
        return receiptAccountMapper.selectList(query);
    }
}
