package cn.iocoder.yudao.module.wealth.service.receiptApproval;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.vo.receiptApproval.ReceiptApprovalCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptApproval.ReceiptApprovalQueryVO;
import cn.iocoder.yudao.module.wealth.vo.receiptApproval.ReceiptApprovalUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 收款单或者收款明细审核 Service 接口
 *
 * @author 系统管理员
 */
public interface ReceiptApprovalService extends IService<ReceiptApprovalDO> {

    /**
     * 创建收款单或者收款明细审核
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createReceiptApproval(@Valid ReceiptApprovalCreateReqVO createReqVO);

    /**
     * 更新收款单或者收款明细审核
     *
     * @param updateReqVO 更新信息
     */
    void updateReceiptApproval(@Valid ReceiptApprovalUpdateReqVO updateReqVO);

    /**
     * 删除收款单或者收款明细审核
     *
     * @param id 编号
     */
    void deleteReceiptApproval(Long id);

    /**
     * 获得收款单或者收款明细审核
     *
     * @param id 编号
     * @return 收款单或者收款明细审核
     */
    ReceiptApprovalDO getReceiptApproval(Long id);

    /**
     * 获得收款单或者收款明细审核列表
     *
     * @param ids 编号
     * @return 收款单或者收款明细审核列表
     */
    List<ReceiptApprovalDO> getReceiptApprovalList(Collection<Long> ids);

    /**
     * 获得收款单或者收款明细审核分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 收款单或者收款明细审核分页
     */
    PageResult<ReceiptApprovalDO> getReceiptApprovalPage(ReceiptApprovalQueryVO query, PageVO page);

    /**
     * 获得收款单或者收款明细审核列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 收款单或者收款明细审核列表
     */
    List<ReceiptApprovalDO> getReceiptApprovalList(ReceiptApprovalQueryVO query);
}
