package cn.iocoder.yudao.module.wealth.service.receiptApproval;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.convert.receiptApproval.ReceiptApprovalConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptApproval.ReceiptApprovalMapper;
import cn.iocoder.yudao.module.wealth.vo.receiptApproval.ReceiptApprovalCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptApproval.ReceiptApprovalQueryVO;
import cn.iocoder.yudao.module.wealth.vo.receiptApproval.ReceiptApprovalUpdateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

/**
 * 收款单或者收款明细审核 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ReceiptApprovalServiceImpl extends AbstractService<ReceiptApprovalMapper, ReceiptApprovalDO> implements ReceiptApprovalService {

    @Resource
    private ReceiptApprovalMapper receiptApprovalMapper;

    @Override
    public Long createReceiptApproval(ReceiptApprovalCreateReqVO createReqVO) {
        // 插入
        ReceiptApprovalDO receiptApproval = ReceiptApprovalConvert.INSTANCE.convert(createReqVO);
        receiptApprovalMapper.insert(receiptApproval);
        // 返回
        return receiptApproval.getId();
    }

    @Override
    public void updateReceiptApproval(ReceiptApprovalUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateReceiptApprovalExists(updateReqVO.getId());
        // 更新
        ReceiptApprovalDO updateObj = ReceiptApprovalConvert.INSTANCE.convert(updateReqVO);
        receiptApprovalMapper.updateById(updateObj);
    }

    @Override
    public void deleteReceiptApproval(Long id) {
        // 校验存在
        this.validateReceiptApprovalExists(id);
        // 删除
        receiptApprovalMapper.deleteById(id);
    }

    private void validateReceiptApprovalExists(Long id) {
        if (receiptApprovalMapper.selectById(id) == null) {
            //throw exception(RECEIPT_APPROVAL_NOT_EXISTS);
        }
    }

    @Override
    public ReceiptApprovalDO getReceiptApproval(Long id) {
        return receiptApprovalMapper.selectById(id);
    }

    @Override
    public List<ReceiptApprovalDO> getReceiptApprovalList(Collection<Long> ids) {
        return receiptApprovalMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceiptApprovalDO> getReceiptApprovalPage(ReceiptApprovalQueryVO query, PageVO page) {
        return receiptApprovalMapper.selectPage(page, query);
    }

    @Override
    public List<ReceiptApprovalDO> getReceiptApprovalList(ReceiptApprovalQueryVO query) {
        return receiptApprovalMapper.selectList(query);
    }
}
