package cn.iocoder.yudao.module.wealth.service.receiptItem;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.io.FileModel;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.BankReceiptDetailsMapper;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemReq;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemResp;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/*
 银行收款明细列表
 */
@Service
@Validated
@Slf4j
public class BankReceiptDetailsImpl implements BankReceiptDetailsService {

    @Resource
    private BankReceiptDetailsMapper bankReceiptDetailsMapper;

    /*
    */
    @Autowired
    private AdminUserService adminUserService;

    /*
     获取分页数据
     */
    public PageResult<ReceiptItemResp> getBankReceiptDetailsList(ReceiptItemReq req) {
        IPage<ReceiptItemResp> mpPage = MyBatisUtils.buildPage(req);
        long total = bankReceiptDetailsMapper.GetCount(req);
        int start = (req.getPageNo() - 1) * req.getPageSize();

        req.setStart(start);
        List<AdminUserDO> ListUser = adminUserService.getUsersList();
        Map<Long, String> userMap =
                ListUser.stream().collect(
                        Collectors.toMap((AdminUserDO::getId), AdminUserDO::getNickname));

        List<ReceiptItemResp> list =
                bankReceiptDetailsMapper.getBankReceiptDetailsList(req);
        for (ReceiptItemResp r : list
        ) {
            if (r.getCreator() == null || r.getCreator().length() == 0)
                r.setCreator("");
            else {
                Long l = Long.parseLong(r.getCreator());
                r.setCreator(userMap.get(l));
            }
            r.setAttrList(getFileList(r.getAttr()));

            if(r.getCreateBpm()==null||r.getCreator().length()==0)
            {
                r.setCreateBpm("");
            }
            else
            {
                Long l2 = Long.parseLong(r.getCreateBpm());
                r.setCreateBpm(userMap.get(l2));
            }
        }
        return new PageResult<>(list, total, mpPage.getSize(),
                req.getPageNo(), (total + mpPage.getSize() - 1) / mpPage.getSize());

    }

    /*
     组装附件信息
     */
    private List<FileModel> getFileList(String sFileStr) {
        if (sFileStr == null || sFileStr.length() == 0) {
            return null;
        } else {
            String[] l = sFileStr.split(",");
            List<FileModel> l1 = new ArrayList<>();
            for (String f : l
            ) {
                FileModel m = new FileModel();
                m.setName(getFileName(f));
                m.setUrl(f);
                l1.add(m);
            }
            return l1;
        }
    }

    /*
     分解Url路径
     */
    public String getFileName(String s) {
        //String s="https://jiedao-test1.oss-cn-shenzhen.aliyuncs.com/2024/05/31/66592b5b556179746796617e.jpg";
        int l = s.lastIndexOf('/');
        String n = s.substring(l + 1);
        return n;
    }
}
