package cn.iocoder.yudao.module.wealth.service.receiptItem;

import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemCancelVo;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemVo;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemUpdateReqVO;
import org.apache.ibatis.annotations.Param;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

/**
 * 收款单明细 Service 接口
 *
 * @author 系统管理员
 */
public interface ReceiptItemService extends IService<ReceiptItemDO> {

    /**
     * 创建收款单明细
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createReceiptItem(@Valid ReceiptItemCreateReqVO createReqVO);

    /**
     * 更新收款单明细
     *
     * @param updateReqVO 更新信息
     */
    void updateReceiptItem(@Valid ReceiptItemUpdateReqVO updateReqVO);

    /**
     * 删除收款单明细
     *
     * @param id 编号
     */
    void deleteReceiptItem(Long id);

    /**
     * 获得收款单明细
     *
     * @param id 编号
     * @return 收款单明细
     */
    ReceiptItemDO getReceiptItem(Long id);

    /**
     * 获得收款单明细列表
     *
     * @param ids 编号
     * @return 收款单明细列表
     */
    List<ReceiptItemDO> getReceiptItemList(Collection<Long> ids);



    /**
     * 核销
     *
     * @param id
     */
    void verification(Long id);

    /**
     * 批量核销
     *
     * @param ids
     */
    void batchVerification(Collection<Long> ids);

    /**
     * 全部核销
     *
     * @param id
     */
    void allVerification(Long id);

    /**
     * 反核销
     *
     * @param id
     */
    void verificationCancel(Long id);


    /**
     * 发起收款明细核销
     *
     * @param financeReceiptItemVo
     */
    void createFinanceReceiptItemWriteOff(FinanceReceiptItemVo financeReceiptItemVo);

    /**
     * 发起收款明细反核销
     *
     * @param financeReceiptItemVo
     */
    void createFinanceReceiptItemWriteOffNo(FinanceReceiptItemVo financeReceiptItemVo);

    /**
     * 取消收款明细核销
     *
     * @param financeReceiptItemVo
     */
    void cancelFinanceReceiptItemWriteOff(FinanceReceiptItemCancelVo financeReceiptItemVo);


    /**
     * 取消收款明细反核销
     *
     * @param financeReceiptItemVo
     */
    void cancelFinanceReceiptItemWriteOffNo(FinanceReceiptItemCancelVo financeReceiptItemVo);


    BigDecimal getWriteOffAmountByReceiptId( Long receiptId);

}
