package cn.iocoder.yudao.module.wealth.service.receiptItem;

import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.wealth.convert.receiptItem.ReceiptItemConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.enums.ReceiptItemStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.ReceiptStatusEnum;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receiptApproval.ReceiptApprovalService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemCancelVo;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemVo;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 收款单明细 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
@Slf4j
public class ReceiptItemServiceImpl extends AbstractService<ReceiptItemMapper, ReceiptItemDO> implements ReceiptItemService {

    @Resource
    private ReceiptItemMapper receiptItemMapper;

    @Resource
    ReceiptService receiptService;

    @Resource
    ReceivableService receivableService;
    @Resource
    DictDataApi dictDataApi;

    @Autowired
    BpmCreateServiceFactory bpmCreateServiceFactory;

    @Autowired
    ReceiptApprovalService receiptApprovalService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createReceiptItem(ReceiptItemCreateReqVO createReqVO) {
        // 插入
        ReceiptItemDO receiptItem = ReceiptItemConvert.INSTANCE.convert(createReqVO);
        receiptItemMapper.insert(receiptItem);
        //创建收款明细的时候要判断收款单状态进行更新
        if (null != createReqVO.getReceiptId()) {
            ReceiptDO receiptDO = receiptService.getReceipt(createReqVO.getReceiptId());
            if (null != receiptDO) {
                if (receiptDO.getState() == 0) {
                    throw exception(RECEIPT_STATUS_NOT_ADD_ITEM);
                } else {
                    if (receiptDO.getState() == 1) {
                        ReceiptDO update = new ReceiptDO();
                        update.setId(receiptDO.getId());
                        update.setState(2);
                        //
                        update.setOrderId(receiptDO.getOrderId());
                        receiptService.updateById(update);
                    }
                }
            } else {
                throw exception(RECEIPT_NOT_EXISTS);
            }
        } else {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        // 返回
        return receiptItem.getId();
    }

    @Override
    public void updateReceiptItem(ReceiptItemUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateReceiptItemExists(updateReqVO.getId());
        // 更新
        ReceiptItemDO updateObj = ReceiptItemConvert.INSTANCE.convert(updateReqVO);
        receiptItemMapper.updateById(updateObj);
    }

    /*
      状态:0:草稿， 1待收款， 2待核销，3已部分核销，4已核销，5已核销待开票，6已开票
      收款单状态
     */
    @Override
    public void deleteReceiptItem(Long id) {
        // 校验存在
        ReceiptItemDO receiptItemDO =
                receiptItemMapper.selectById(id);

        if (null == receiptItemDO) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
        // 删除
        receiptItemMapper.deleteById(id);
        //判断当前收款单下还有没有收款明细
        if (null != receiptItemDO.getReceiptId()) {
            LambdaQueryWrapper<ReceiptItemDO> queryWrapper =
                    new LambdaQueryWrapper<>();
            queryWrapper.eq(ReceiptItemDO::getReceiptId,
                    receiptItemDO.getReceiptId());
            long count = this.count(queryWrapper);
            if (count == 0) {
                ReceivableDO updateReceivableDO = new ReceivableDO();
                updateReceivableDO.setState(0);
                //更新当前收款单下的应收明细为待收款
                UpdateWrapper updateWrapper = new UpdateWrapper();
                updateWrapper.eq("receipt_id", receiptItemDO.getReceiptId());
                this.receivableService.update(updateReceivableDO, updateWrapper);

                //添加的逻辑点
                //当银行收款明细为0是要改变收款单的状态为 待收款
                ReceiptDO update = new ReceiptDO();
                update.setId(receiptItemDO.getReceiptId());
                update.setState(1);
                receiptService.updateById(update);
                //end 添加的逻辑点
            }
        }

    }

    private void validateReceiptItemExists(Long id) {
        if (receiptItemMapper.selectById(id) == null) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
    }

    @Override
    public ReceiptItemDO getReceiptItem(Long id) {
        return receiptItemMapper.selectById(id);
    }

    @Override
    public List<ReceiptItemDO> getReceiptItemList(Collection<Long> ids) {
        return receiptItemMapper.selectBatchIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void verification(Long id) {
        ReceiptItemDO receiptItemDO = this.getReceiptItem(id);
        if (receiptItemDO == null) {
            throw exception(RECEIPTITEM_NOT_EXISTS);
        }

        if (receiptItemDO.getStatus() != 0) {
            throw exception(RECEIPTITEM_VERIFICATION);
        }

        ReceiptItemDO updateReceiptItemDO = new ReceiptItemDO();
        updateReceiptItemDO.setId(receiptItemDO.getId());
        updateReceiptItemDO.setStatus(1);
        this.updateById(updateReceiptItemDO);

        if (null == receiptItemDO.getReceiptId()) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        ReceiptDO receiptDO = receiptService.getById(receiptItemDO.getReceiptId());
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        if (receiptDO.getState() == 0) {
            throw exception(RECEIPT_STATUS_NOT_WRITE_OFF);
        }


        //设置应收明细为收款中
        LambdaUpdateWrapper<ReceivableDO> lambdaReceivableUpdateWrapper = new LambdaUpdateWrapper();
        lambdaReceivableUpdateWrapper.set(ReceivableDO::getState, 1);
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptDO.getId());
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getState, 0);
        receivableService.update(lambdaReceivableUpdateWrapper);


        //需要计算出核销比例
        BigDecimal writeOffAmount = receiptItemMapper.getWriteOffAmountByReceiptId(receiptDO.getId());

        ReceiptDO updateReceiptDO = new ReceiptDO();
        updateReceiptDO.setId(receiptDO.getId());
        //需要计算出核销比例
        if (receiptDO.getState() == 2) {
            updateReceiptDO.setState(3);
        }

        if (null != receiptDO.getReceivableTotalAmount() && receiptDO.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) == 1) {
            updateReceiptDO.setWriteOffProportion(writeOffAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP));
        }

        receiptService.updateById(updateReceiptDO);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchVerification(Collection<Long> ids) {
        Long receiptId = null;
        for (Long id : ids) {
            ReceiptItemDO receiptItemDO = this.getReceiptItem(id);
            if (null == receiptItemDO) {
                throw exception(RECEIPTITEM_NOT_EXISTS);
            }
            receiptId = receiptItemDO.getReceiptId();
            if (receiptItemDO.getStatus() == 0) {
                ReceiptItemDO updateReceiptDO = new ReceiptItemDO();
                updateReceiptDO.setId(receiptItemDO.getId());

                updateReceiptDO.setStatus(1);
                this.updateById(updateReceiptDO);
            } else {
                throw exception(RECEIPTITEM_VERIFICATION);
            }
        }


        if (null == receiptId) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        ReceiptDO receiptDO = receiptService.getById(receiptId);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        //设置应收明细为收款中
        LambdaUpdateWrapper<ReceivableDO> lambdaReceivableUpdateWrapper = new LambdaUpdateWrapper();
        lambdaReceivableUpdateWrapper.set(ReceivableDO::getState, 1);
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getReceiptId, receiptId);
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getState, 0);
        receivableService.update(lambdaReceivableUpdateWrapper);

        //需要计算出核销比例
        BigDecimal writeOffAmount = receiptItemMapper.getWriteOffAmountByReceiptId(receiptDO.getId());

        ReceiptDO updateReceiptDO = new ReceiptDO();
        updateReceiptDO.setId(receiptDO.getId());
        //需要计算出核销比例
        if (receiptDO.getState() == 2) {
            updateReceiptDO.setState(3);
        }

        if (null != receiptDO.getReceivableTotalAmount() && receiptDO.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) == 1) {
            updateReceiptDO.setWriteOffProportion(writeOffAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP));
        }

        receiptService.updateById(updateReceiptDO);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void allVerification(Long id) {
        //进行验证判断核销是否达到100%
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptItemDO::getStatus, 1);
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, id);
        List<ReceiptItemDO> receiptItemDOList = this.list(lambdaQueryWrapper);
        if (null == receiptItemDOList || receiptItemDOList.size() == 0) {
            throw exception(RECEIPT_NOT_WRITE_OFF_ITEM_NOT_EXISTS);
        }

        BigDecimal totalAmount = receiptItemDOList.stream().map(ReceiptItemDO::getWriteOffAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        //核销全部收款明细的时候要判断收款单状态进行更新
        ReceiptDO receiptDO = receiptService.getReceipt(id);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        if (receiptDO.getState() != 2 && receiptDO.getState() != 3) {
            throw exception(RECEIPT_WRITE_OFF_STATUS_NOT);
        }


        Long currencyId = receiptDO.getCurrencyId();
        if (null == currencyId) {
            throw exception(RECEIPT_CURRENCY_NOT_EXISTS);
        }
        if (totalAmount.compareTo(receiptDO.getReceivableTotalAmount()) == -1) {
            DictDataRespDTO dictDataRespDTO = dictDataApi.parseDictDataFromCache("write_off_diff", String.valueOf(currencyId));
            if (null == dictDataRespDTO) {
                throw exception(RECEIPT_CURRENCY_DIFF_NOT_EXISTS);
            }
            //核销差额大于配置提示
            if (receiptDO.getReceivableTotalAmount().subtract(totalAmount).compareTo(new BigDecimal(dictDataRespDTO.getValue())) == 1) {
                throw exception(RECEIPT_CURRENCY_DIFF_BIG);
            }
        }

        //设置应收明细为已收款状态
        LambdaUpdateWrapper<ReceivableDO> lambdaReceivableUpdateWrapper = new LambdaUpdateWrapper();
        lambdaReceivableUpdateWrapper.set(ReceivableDO::getState, 2);
        lambdaReceivableUpdateWrapper.eq(ReceivableDO::getReceiptId, id);
        receivableService.update(lambdaReceivableUpdateWrapper);


        LambdaUpdateWrapper<ReceiptItemDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
        lambdaUpdateWrapper.set(ReceiptItemDO::getStatus, 1);
        lambdaUpdateWrapper.eq(ReceiptItemDO::getReceiptId, id);
        lambdaUpdateWrapper.eq(ReceiptItemDO::getStatus, 0);
        this.update(lambdaUpdateWrapper);

        //需要计算出核销比例
        ReceiptDO update = new ReceiptDO();
        update.setId(receiptDO.getId());
        if (receiptDO.getOpenInvoice() == 1) {
            update.setState(5);
        } else {
            update.setState(4);
        }
        update.setWriteOffAt(new Date());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (null != loginUser) {
            update.setWriteOffId(loginUser.getId());
            update.setWriteOffName(loginUser.getNickname());
        }
        update.setWriteOffProportion(new BigDecimal("100"));//设置核销比例
        receiptService.updateById(update);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void verificationCancel(Long id) {
        ReceiptItemDO receiptItemDO = this.getReceiptItem(id);
        if (receiptItemDO == null) {
            throw exception(RECEIPT_ITEM_NOT_EXISTS);
        }
        if (receiptItemDO.getStatus() != 1) {
            throw exception(RECEIPTITEM_VERIFICATION);
        }

        ReceiptItemDO updateReceiptItemDO = new ReceiptItemDO();
        updateReceiptItemDO.setId(receiptItemDO.getId());
        updateReceiptItemDO.setStatus(0);
        this.updateById(updateReceiptItemDO);

        if (null == receiptItemDO.getReceiptId()) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        ReceiptDO receiptDO = receiptService.getById(receiptItemDO.getReceiptId());
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }

        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, receiptItemDO.getReceiptId());
        lambdaQueryWrapper.eq(ReceiptItemDO::getStatus, 1);
        long writeCount = this.count(lambdaQueryWrapper);

        ReceiptDO updateReceiptDO = new ReceiptDO();
        updateReceiptDO.setId(receiptDO.getId());
        if (writeCount == 0) {
            updateReceiptDO.setState(2);
        } else {
            updateReceiptDO.setState(3);
        }

        //需要计算出核销比例

        BigDecimal writeOffAmount = receiptItemMapper.getWriteOffAmountByReceiptId(receiptItemDO.getReceiptId());

        log.error("------------------------------{}", writeOffAmount.toString());

        if (null != receiptDO.getReceivableTotalAmount() && receiptDO.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) == 1) {
            updateReceiptDO.setWriteOffProportion(writeOffAmount.divide(receiptDO.getReceivableTotalAmount(), 8, RoundingMode.HALF_UP).multiply(new BigDecimal("100")).setScale(6, BigDecimal.ROUND_HALF_UP));
        }

        receiptService.updateById(updateReceiptDO);

        //TODO 收款单明细反核销需要发起流程
    }

    @Override
    public void createFinanceReceiptItemWriteOff(FinanceReceiptItemVo financeReceiptItemVo) {
        ReceiptItemDO receiptItemDO = this.getById(financeReceiptItemVo.getReceiptItemId());
        if (null == receiptItemDO) {
            throw exception(RECEIPT_ITEM_NOT_FOUND);
        }
        if (receiptItemDO.getStatus() == 1) {
            throw exception(RECEIPT_ITEM_STATE_WRITE_OFF_SUCCESS_NOT_APPROVE);
        }


        //校验收款单是否在在全部核销中
        validateReceipt(receiptItemDO.getReceiptId());


        //创建收款单明细核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptItemVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptItemVo.getReceiptNo());
        approvalDO.setReceiptItemId(financeReceiptItemVo.getReceiptItemId());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptItemVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF.getKey(), financeReceiptItemVo.getReceiptNo(), financeReceiptItemVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptItemDO update = new ReceiptItemDO();
        update.setId(financeReceiptItemVo.getReceiptItemId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setStatus(ReceiptItemStatusEnum.WRITE_OFF_APPROVE_ING.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);
    }

    @Override
    public void createFinanceReceiptItemWriteOffNo(FinanceReceiptItemVo financeReceiptItemVo) {
        //如果收款单状态是全部核销中或者全部核销通过不能发起反核销
        ReceiptItemDO receiptItemDO = this.getById(financeReceiptItemVo.getReceiptItemId());
        if (null == receiptItemDO) {
            throw exception(RECEIPT_ITEM_NOT_FOUND);
        }
        if (receiptItemDO.getStatus() == 0) {
            throw exception(RECEIPT_ITEM_STATE_WRITE_OFF__NOT_APPROVE);
        }
        if (null == receiptItemDO.getReceiptId()) {
            throw exception(RECEIPT_NOT_FOUND);
        }
        //校验收款单是否在在全部核销中
        validateReceipt(receiptItemDO.getReceiptId());

        ReceiptDO receiptDO = receiptService.getReceipt(receiptItemDO.getReceiptId());
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_FOUND);
        }


        if (ReceiptStatusEnum.WRITE_OFF_ALL_ING.getValue() == receiptDO.getState()) {
            throw exception(RECEIPT_WRITE_OFF_ALL_ING_ITEM_NOT_APPROVE);
        }
        if (ReceiptStatusEnum.WRITE_OFF_ALL_SUCCESS.getValue() == receiptDO.getState()) {
            throw exception(RECEIPT_WRITE_OFF_ALL_SUCCESS_ITEM_NOT_APPROVE);
        }
        if (ReceiptStatusEnum.WRITE_OFF_ALL_OPENING.getValue() == receiptDO.getState()) {
            throw exception(RECEIPT_WRITE_OFF_ALL_SUCCESS_ITEM_NOT_APPROVE);
        }
        if (ReceiptStatusEnum.OPEN_BILL.getValue() == receiptDO.getState()) {
            throw exception(RECEIPT_WRITE_OFF_ALL_SUCCESS_ITEM_NOT_APPROVE);
        }


        //创建收款单明细反核销流程
        ReceiptApprovalDO approvalDO = new ReceiptApprovalDO();
        approvalDO.setReceiptId(financeReceiptItemVo.getReceiptId());
        approvalDO.setReceiptNo(financeReceiptItemVo.getReceiptNo());
        approvalDO.setReceiptItemId(financeReceiptItemVo.getReceiptItemId());
        approvalDO.setBmpKey(WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF_NO.getKey());
        approvalDO.setStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        approvalDO.setReason(financeReceiptItemVo.getRemark());
        receiptApprovalService.save(approvalDO);
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmId = bpmCreateServiceFactory.createBmp(userId, approvalDO.getId(), WorkFlowEmus.FINANCE_RECEIPT_ITEM_WRITE_OFF_NO.getKey(), financeReceiptItemVo.getReceiptNo(), financeReceiptItemVo.getCopyUserList());
        approvalDO.setBmpId(bpmId);
        receiptApprovalService.updateById(approvalDO);

        ReceiptItemDO update = new ReceiptItemDO();
        update.setId(financeReceiptItemVo.getReceiptItemId());
        update.setBmpStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        update.setStatus(ReceiptItemStatusEnum.WRITE_OFF_NO_APPROVE_ING.getValue());
        update.setBmpId(bpmId);
        this.updateById(update);
    }

    @Override
    public void cancelFinanceReceiptItemWriteOff(FinanceReceiptItemCancelVo financeReceiptItemVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptItemDO receiptItemDO = validateReceiptItemBmpCancel(financeReceiptItemVo.getReceiptItemId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptItemDO.getBmpId(), financeReceiptItemVo.getRemark());
    }

    @Override
    public void cancelFinanceReceiptItemWriteOffNo(FinanceReceiptItemCancelVo financeReceiptItemVo) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ReceiptItemDO receiptItemDO = validateReceiptItemBmpCancel(financeReceiptItemVo.getReceiptItemId());
        bpmCreateServiceFactory.cancelBmpById(userId, receiptItemDO.getBmpId(), financeReceiptItemVo.getRemark());
    }

    private ReceiptItemDO validateReceiptItemBmpCancel(Long id) {
        ReceiptItemDO receiptItemDO = this.getReceiptItem(id);
        if (null == receiptItemDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        }
        if (null == receiptItemDO.getBmpStatus()) {

            throw exception(BMP_NOT);
        }

        if (receiptItemDO.getBmpStatus() != BpmProcessInstanceResultEnum.PROCESS.getResult()) {
            throw exception(BMP_CANCEL);
        }

        return receiptItemDO;
    }

    private void validateReceipt(Long receiptId) {
        ReceiptDO receiptDO = receiptService.getById(receiptId);
        if (null == receiptDO) {
            throw exception(RECEIPT_NOT_EXISTS);
        } else {
            if (BpmProcessInstanceResultEnum.PROCESS.getResult() == receiptDO.getBmpStatus()) {
                throw exception(RECEIPT_ALL_WRITE_OFF_ING);
            }
        }
    }


    public BigDecimal getWriteOffAmountByReceiptId(Long receiptId) {
        return receiptItemMapper.getWriteOffAmountByReceiptId(receiptId);
    }
}
