package cn.iocoder.yudao.module.wealth.service.receivableDiscount;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivableDiscount.ReceivableDiscountDO;
import cn.iocoder.yudao.module.wealth.vo.receivableDiscount.ReceivableDiscountCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receivableDiscount.ReceivableDiscountQueryVO;
import cn.iocoder.yudao.module.wealth.vo.receivableDiscount.ReceivableDiscountUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 应收款优惠信息 Service 接口
 *
 * @author 系统管理员
 */
public interface ReceivableDiscountService extends IService<ReceivableDiscountDO> {

    /**
     * 创建应收款优惠信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createReceivableDiscount(@Valid ReceivableDiscountCreateReqVO createReqVO);

    /**
     * 更新应收款优惠信息
     *
     * @param updateReqVO 更新信息
     */
    void updateReceivableDiscount(@Valid ReceivableDiscountUpdateReqVO updateReqVO);

    /**
     * 删除应收款优惠信息
     *
     * @param id 编号
     */
    void deleteReceivableDiscount(Long id);

    /**
     * 获得应收款优惠信息
     *
     * @param id 编号
     * @return 应收款优惠信息
     */
    ReceivableDiscountDO getReceivableDiscount(Long id);

    /**
     * 获得应收款优惠信息列表
     *
     * @param ids 编号
     * @return 应收款优惠信息列表
     */
    List<ReceivableDiscountDO> getReceivableDiscountList(Collection<Long> ids);

    /**
     * 获得应收款优惠信息分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 应收款优惠信息分页
     */
    PageResult<ReceivableDiscountDO> getReceivableDiscountPage(ReceivableDiscountQueryVO query, PageVO page);

    /**
     * 获得应收款优惠信息列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 应收款优惠信息列表
     */
    List<ReceivableDiscountDO> getReceivableDiscountList(ReceivableDiscountQueryVO query);
}
