package cn.iocoder.yudao.module.wealth.service.receivableDiscount;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.convert.receivableDiscount.ReceivableDiscountConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivableDiscount.ReceivableDiscountDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivableDiscount.ReceivableDiscountMapper;
import cn.iocoder.yudao.module.wealth.vo.receivableDiscount.ReceivableDiscountCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receivableDiscount.ReceivableDiscountQueryVO;
import cn.iocoder.yudao.module.wealth.vo.receivableDiscount.ReceivableDiscountUpdateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.RECEIVABLE_DISCOUNT_NOT_EXISTS;


/**
 * 应收款优惠信息 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ReceivableDiscountServiceImpl extends AbstractService<ReceivableDiscountMapper, ReceivableDiscountDO> implements ReceivableDiscountService {

    @Resource
    private ReceivableDiscountMapper receivableDiscountMapper;

    @Override
    public Long createReceivableDiscount(ReceivableDiscountCreateReqVO createReqVO) {
        // 插入
        ReceivableDiscountDO receivableDiscount = ReceivableDiscountConvert.INSTANCE.convert(createReqVO);
        receivableDiscountMapper.insert(receivableDiscount);
        // 返回
        return receivableDiscount.getId();
    }

    @Override
    public void updateReceivableDiscount(ReceivableDiscountUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateReceivableDiscountExists(updateReqVO.getId());
        // 更新
        ReceivableDiscountDO updateObj = ReceivableDiscountConvert.INSTANCE.convert(updateReqVO);
        receivableDiscountMapper.updateById(updateObj);
    }

    @Override
    public void deleteReceivableDiscount(Long id) {
        // 校验存在
        this.validateReceivableDiscountExists(id);
        // 删除
        receivableDiscountMapper.deleteById(id);
    }

    private void validateReceivableDiscountExists(Long id) {
        if (receivableDiscountMapper.selectById(id) == null) {
            throw exception(RECEIVABLE_DISCOUNT_NOT_EXISTS);
        }
    }

    @Override
    public ReceivableDiscountDO getReceivableDiscount(Long id) {
        return receivableDiscountMapper.selectById(id);
    }

    @Override
    public List<ReceivableDiscountDO> getReceivableDiscountList(Collection<Long> ids) {
        return receivableDiscountMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceivableDiscountDO> getReceivableDiscountPage(ReceivableDiscountQueryVO query, PageVO page) {
        return receivableDiscountMapper.selectPage(page, query);
    }

    @Override
    public List<ReceivableDiscountDO> getReceivableDiscountList(ReceivableDiscountQueryVO query) {
        return receivableDiscountMapper.selectList(query);
    }
}
