package cn.iocoder.yudao.module.wealth.vo.payable;

import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.wealth.enums.BoxAirStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.BoxApprovalTypeEnum;
import cn.iocoder.yudao.module.wealth.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.wealth.enums.TransportTypeEnum;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import java.util.Date;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
 * 应付款 Response VO
 *
 * @author 郑屹
 */
@Data
@ApiModel("管理后台 - 应付款 Response VO")
public class PayableBackVO {

    @ExcelProperty("主键")
    @ApiModelProperty(value = "主键", required = true)
    private Long id;

    @ExcelProperty("自编号")
    @ApiModelProperty(value = "自编号")
    private String payableNo;

    @ExcelProperty("付款单id")
    @ApiModelProperty(value = "付款单id")
    private Long paymentId;

    @ExcelProperty("付款单号")
    @ApiModelProperty(value = "付款单号")
    private String paymentNo;

    @ExcelProperty("订单id")
    @ApiModelProperty(value = "订单id")
    private Long orderId;

    @ExcelProperty("订单号")
    @ApiModelProperty(value = "订单号")
    private String orderNo;

    @ExcelProperty("供应商id")
    @ApiModelProperty(value = "供应商id")
    private Long supplierId;

    @ExcelProperty("供应商名称")
    @ApiModelProperty(value = "供应商名称")
    private String supplierName;

    @ExcelProperty("费用类型(字典:payable_fee_type)")
    @ApiModelProperty(value = "费用类型(字典:payable_fee_type)", required = true)
    private Long feeType;

    @ExcelProperty("金额")
    @ApiModelProperty(value = "金额")
    private java.math.BigDecimal totalAmount;

    @ExcelProperty("币货币id")
    @ApiModelProperty(value = "币货币id")
    private Long currencyId;

    @ExcelProperty("备注")
    @ApiModelProperty(value = "备注")
    private String remark;

    @ExcelProperty("汇率")
    @ApiModelProperty(value = "汇率")
    private String exchangeRate;

    @ExcelProperty("税率")
    @ApiModelProperty(value = "税率")
    private java.math.BigDecimal taxRate;

    @ExcelProperty("税价合计")
    @ApiModelProperty(value = "税价合计")
    private java.math.BigDecimal total;

    @ExcelProperty("收款状态（0未付款，1付款中，2已付款）")
    @ApiModelProperty(value = "收款状态（0未付款，1付款中，2已付款）")
    private Integer state;

    @ExcelProperty("数据来源（pc端，mobile端）")
    @ApiModelProperty(value = "数据来源（pc端，mobile端）")
    private String source;

    @ExcelProperty("操作步骤")
    @ApiModelProperty(value = "操作步骤")
    private Long stepsId;

    @ExcelProperty("额外费用")
    @ApiModelProperty(value = "额外费用")
    private Integer extraCost;

    @ExcelProperty("货柜id")
    @ApiModelProperty(value = "货柜id")
    private Long containerId;

    @ExcelProperty("货柜号")
    @ApiModelProperty(value = "货柜号")
    private String containerNo;

    @ExcelProperty("发票号")
    @ApiModelProperty(value = "发票号", required = true)
    private String invoiceNumber;

    @ExcelProperty("创建时间")
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY)
    @ApiModelProperty(value = "创建时间", required = true)
    private Date createTime;

    @ApiModelProperty(value = "始发地")
    private Long startWarehouseId;

    @ApiModelProperty(value = "目的地")
    private Long destWarehouseId;

    @ApiModelProperty(value = "运输方式")
    private Long transportId;

    @ApiModelProperty(value = "出货费ID")
    private Long shippingFeeId;

    /**
     * 最后更新时间
     */
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY)
    private Date updateTime;


    private String creator;
    /**
     * 更新者，目前使用 SysUser 的 id 编
     **/
    private String updater;


    /*
     运输方式
     */
    private  String transportTypeName;

    @ApiModelProperty(value = "正在反审的类型")
    private Integer backApprovalType;

    private String shipmentStatusText;

    @ApiModelProperty(value = "出货状态")
    private Integer shipmentStatus;

    @ApiModelProperty(value = "空运出货状态")
    private Integer shipmentStatusAir;

    /*
   费用产生时间
    日期格式转换
   */
    @ApiModelProperty(value = "费用产生日期")
    @NotNull(message = "费用产生日期不能为空")
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY)
    private Date freecsdate;

    public String getShipmentStatusText() {
        if (backApprovalType != null) {
            shipmentStatusText = BoxApprovalTypeEnum.getApprovalTypeByTypeValue(backApprovalType).getText();
        } else {
            if (TransportTypeEnum.AIR.getType().equals(transportId)) {
                if (null != shipmentStatusAir) {
                    shipmentStatusText = I18nMessage.getMessage(BoxAirStatusEnum.getStatusEnumByStatus(shipmentStatusAir).getText());
                }
            } else {
                if (null != shipmentStatus) {
                    shipmentStatusText = I18nMessage.getMessage(BoxStatusEnum.getStatusEnumByStatus(shipmentStatus).getText());
                }
            }
        }
        return shipmentStatusText;
    }

    public void setShipmentStatusText(String shipmentStatusText) {
        this.shipmentStatusText = shipmentStatusText;
    }


}
