package cn.iocoder.yudao.module.wealth.vo.payable;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.*;

import java.util.*;

import io.swagger.annotations.*;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.*;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
 * 应付款 Base VO，提供给添加、修改、详细的子 VO 使用
 * 如果子 VO 存在差异的字段，请不要添加到这里，影响 Swagger 文档生成
 */
@Data
public class PayableBaseVO {

    @ApiModelProperty(value = "自编号")
    private String payableNo;

    @ApiModelProperty(value = "付款单id")
    private Long paymentId;

    @ApiModelProperty(value = "付款单号")
    private String paymentNo;

    @ApiModelProperty(value = "订单id")
    private Long orderId;

    @ApiModelProperty(value = "订单号")
    private String orderNo;

    @ApiModelProperty(value = "供应商id")
    private Long supplierId;

    @ApiModelProperty(value = "供应商名称")
    private String supplierName;

    @ApiModelProperty(value = "费用类型(字典:payable_fee_type)", required = true)
    @NotNull(message = "费用类型(字典:payable_fee_type)不能为空")
    private Long feeType;

    @ApiModelProperty(value = "金额")
    private java.math.BigDecimal totalAmount;

    @ApiModelProperty(value = "币货币id")
    private Long currencyId;

    @ApiModelProperty(value = "备注")
    private String remark;

    @ApiModelProperty(value = "汇率")
    private String exchangeRate;

    @ApiModelProperty(value = "税率")
    private java.math.BigDecimal taxRate;

    @ApiModelProperty(value = "税价合计")
    private java.math.BigDecimal total;

    @ApiModelProperty(value = "收款状态（0未付款，1付款中，2已付款）")
    private Integer state;

    @ApiModelProperty(value = "数据来源（pc端，mobile端）")
    private String source;

    @ApiModelProperty(value = "操作步骤")
    private Long stepsId;

    @ApiModelProperty(value = "额外费用")
    private Integer extraCost;

    @ApiModelProperty(value = "货柜id")
    private Long containerId;

    @ApiModelProperty(value = "货柜号")
    private String containerNo;

    @ApiModelProperty(value = "发票号", required = true)
//    @NotNull(message = "发票号不能为空")
    private String invoiceNumber;

    @ApiModelProperty(value = "始发地")
    private Long startWarehouseId;

    @ApiModelProperty(value = "目的地")
    private Long destWarehouseId;

    @ApiModelProperty(value = "运输方式")
    private Long transportId;

    @ApiModelProperty(value = "出货费ID")
    private Long shippingFeeId;


    /*
    费用产生时间
     一定要加格式转换注解，不然前端过来的字符串就会报异常
    */
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY)
    private Date freecsdate;

}
