package cn.iocoder.yudao.module.wealth.vo.payment;

import cn.iocoder.yudao.framework.common.util.io.FileModel;
import com.alibaba.excel.annotation.ExcelProperty;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
 * 付款单 Response VO
 *
 * @author 郑屹
 */
@Data
@ApiModel("管理后台 - 付款单 Response VO")
public class PaymentBackVO {

    @ExcelProperty("主键")
    @ApiModelProperty(value = "主键", required = true)
    private Long id;

    @ExcelProperty("付款单号")
    @ApiModelProperty(value = "付款单号")
    private String paymentNo;

    @ExcelProperty("部门Id")
    @ApiModelProperty(value = "部门Id")
    private Long departmentId;

    @ExcelProperty("部门名称")
    @ApiModelProperty(value = "部门名称")
    private String departmentName;

    @ExcelProperty("业务员")
    @ApiModelProperty(value = "业务员")
    private Long salesmanId;

    @ExcelProperty("业务员名称")
    @ApiModelProperty(value = "业务员名称", required = true)
    private String salesmanName;

    @ExcelProperty("供应商id")
    @ApiModelProperty(value = "供应商id")
    private Long supplierId;

    @ExcelProperty("供应商名称")
    @ApiModelProperty(value = "供应商名称")
    private String supplierName;

    @ApiModelProperty(value = "银行名称")
    private String supplierBank;

    @ApiModelProperty(value = "银行账号")
    private String supplierBankAccount;

    @ApiModelProperty(value = "银行账户名称")
    private String supplierBankAccountName;

    @ExcelProperty("申请日期")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "申请日期")
    private Date applicationAt;

    @ExcelProperty("最迟付款日")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "最迟付款日")
    private Date latestPayAt;

    @ExcelProperty("账单号")
    @ApiModelProperty(value = "账单号")
    private String accountNumber;

    @ExcelProperty("发票状态（0未开票，1已开票）")
    @ApiModelProperty(value = "发票状态（0未开票，1已开票）")
    private Integer invoiceStatus;

    @ExcelProperty("发票号")
    @ApiModelProperty(value = "发票号")
    private String invoiceNumber;

    @ExcelProperty("付款单状态（0草稿 1审核中，2审批驳回，3反审核中，4待核销 5核销审核中 6已核销 7反核销审核中）")
    @ApiModelProperty(value = "付款单状态（0草稿 1审核中，2审批驳回，3反审核中，4待核销 5核销审核中 6已核销 7反核销审核中）")
    private Integer state;

    @ExcelProperty("付款单添加人id")
    @ApiModelProperty(value = "付款单添加人id")
    private Long paymentAddId;

    @ExcelProperty("付款单添加人名称")
    @ApiModelProperty(value = "付款单添加人名称")
    private String paymentAddBy;

    @ExcelProperty("审核人id")
    @ApiModelProperty(value = "审核人id")
    private Long reviewId;

    @ExcelProperty("审核人名称")
    @ApiModelProperty(value = "审核人名称")
    private String reviewName;

    @ExcelProperty("核销人id")
    @ApiModelProperty(value = "核销人id")
    private Long writeOffId;

    @ExcelProperty("核销人名称")
    @ApiModelProperty(value = "核销人名称")
    private String writeOffName;

    @ExcelProperty("审核日期")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "审核日期")
    private Date reviewAt;

    @ExcelProperty("核销日期")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "核销日期")
    private Date writeOffAt;

    @ExcelProperty("数据来源（pc端，mobile端）")
    @ApiModelProperty(value = "数据来源（pc端，mobile端）")
    private String source;

    @ExcelProperty("结算方式")
    @ApiModelProperty(value = "结算方式")
    private Integer settlementType;

    @ExcelProperty("创建时间")
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "创建时间", required = true)
    private Date createTime;

    @ApiModelProperty(value = "当前流程ID")
    private String bmpId;

    @ApiModelProperty(value = "驳回原因")
    private String comment;

    /*
  实际付款时间
  */
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY)
    private Date sjfkdate;

    /*
  原来的付款备注没有保存
  */
    @ApiModelProperty(value = "付款备注")
    private String reason;


    private String fkren;

    /*
     核销时间
     */
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY)
    private  Date hexiaodate;

    /*
     编辑付款单时的备注
     */
    private String editreason;

    /*
     文件列表
     */
    private List<FileModel> filelist;

}
