package cn.iocoder.yudao.module.wealth.vo.receivable;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

/**
 * 应收款 Base VO，提供给添加、修改、详细的子 VO 使用
 * 如果子 VO 存在差异的字段，请不要添加到这里，影响 Swagger 文档生成
 */
@Data
public class ReceivableBaseVO {

    @ApiModelProperty(value = "收款单id")
    private Long receiptId;

    @ApiModelProperty(value = "收款单号")
    private String receiptNo;

    @ApiModelProperty(value = "订单id")
    @NotNull(message = "订单ID不能为空")
    private Long orderId;

    @ApiModelProperty(value = "订单号")
    @NotBlank(message = "订单号不能为空")
    private String orderNo;

    @ApiModelProperty(value = "订单项商品ID")
    private Long orderItemId;

    @ApiModelProperty(value = "中文标题")
    private String titleZh;

    @ApiModelProperty(value = "英文标题")
    private String titleEn;

    @ApiModelProperty(value = "数量")
    private Integer quantity;

    @ApiModelProperty(value = "箱数")
    private Integer num;

    @ApiModelProperty(value = "体积/重量")
    private BigDecimal weight;

    @ApiModelProperty(value = "体积")
    private BigDecimal volume;

    @ApiModelProperty(value = "收款类型(字典 payment_type)")
    private Integer collectionType;

    @ApiModelProperty(value = "费用类型(字典 receivable_fee_type)")
    private Integer feeType;

    @ApiModelProperty(value = "汇率")
    private BigDecimal exchangeRate;

    @ApiModelProperty(value = "货币id")
    private Long currencyId;

    @ApiModelProperty(value = "单价")
    private BigDecimal unitPrice;

    @ApiModelProperty(value = "总金额")
    private BigDecimal totalAmount;

    @ApiModelProperty(value = "优惠金额")
    private BigDecimal discountTotal;

    @ApiModelProperty(value = "优惠备注")
    @Length(max = 200, message = "优惠备注最大长度不能超过200")
    private String discountRemark;

    @ApiModelProperty(value = "是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    @ApiModelProperty(value = "收款状态（0未收款，1收款中，2已收款）")
    private Integer state;

    @ApiModelProperty(value = "数据来源（pc端，mobile端）")
    private String source;

    @ApiModelProperty(value = "作者")
    private String author;

    @ApiModelProperty(value = "货值")
    private BigDecimal amountWorth;

    @ApiModelProperty(value = "付款人")
    private String paymentUser;

    @ApiModelProperty(value = "费用来源：1 订单计算 2 费用申请 3 特需费用")
    private Integer feeSource;

    @ApiModelProperty(value = "税点")
    private BigDecimal taxPoint;

    @ApiModelProperty(value = "含税金额")
    private BigDecimal taxAmount;

    @ApiModelProperty(value = "备注")
    private String remark;

}
