package cn.iocoder.yudao.module.wealth.vo.receivable;

import com.alibaba.excel.annotation.ExcelProperty;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Data
@ApiModel("管理后台 - 应收款查询 VO")
public class ReceivableQueryVO {

    @ApiModelProperty(value = "收款单id")
    private Long receiptId;

    @ApiModelProperty(value = "收款单号")
    private String receiptNo;

    @ApiModelProperty(value = "订单id")
    private Long orderId;

    @ApiModelProperty(value = "订单号")
    private String orderNo;

    @ApiModelProperty(value = "订单项商品ID")
    private Long orderItemId;

    @ApiModelProperty(value = "中文标题")
    private String titleZh;

    @ApiModelProperty(value = "英文标题")
    private String titleEn;

    @ApiModelProperty(value = "数量")
    private Integer quantity;

    @ApiModelProperty(value = "箱数")
    private Integer num;

    @ApiModelProperty(value = "体积/重量")
    private BigDecimal weight;

    @ApiModelProperty(value = "体积")
    private BigDecimal volume;

    @ApiModelProperty(value = "费用类型(字典 receivable_fee_type)")
    private Integer feeType;

    @ApiModelProperty(value = "汇率")
    private BigDecimal exchangeRate;

    @ApiModelProperty(value = "货币id")
    private Long currencyId;

    @ApiModelProperty(value = "单价")
    private BigDecimal unitPrice;

    @ApiModelProperty(value = "总金额")
    private BigDecimal totalAmount;

    @ApiModelProperty(value = "优惠金额")
    private BigDecimal discountTotal;

    @ApiModelProperty(value = "优惠备注")
    private BigDecimal discountRemark;

    @ApiModelProperty(value = "是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    @ApiModelProperty(value = "收款状态（0未收款，1收款中，2已收款）")
    private Integer state;

    @ApiModelProperty(value = "数据来源（pc端，mobile端）")
    private String source;

    @ApiModelProperty(value = "作者")
    private String author;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始创建时间")
    private Date beginCreateTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束创建时间")
    private Date endCreateTime;

    @ApiModelProperty(value = "国际化")
    private Integer lang;

    @ApiModelProperty(value = "运输方式id")
    private Integer transportId;

    @ApiModelProperty(value = "是否控货")
    private Boolean isCargoControl;

    @ApiModelProperty(value = "报关类别：我司全代：1，自单代报：2，混合报关：3")
    private Integer customsType;

    @ApiModelProperty(value = "始发地id")
    private Long departureId;

    @ApiModelProperty(value = "目的地id")
    private Long objectiveId;

    @ApiModelProperty(value = "唛头")
    private String marks;

    @ApiModelProperty(value = "订单状态详情见字典：order_status")
    private Integer status;

    @ApiModelProperty(value = "订单异常状态(字典 order_abnormal_state)")
    private Integer abnormalState;

    @ApiModelProperty(value = "订单入仓状态(字典 order_warehouse_in_status)")
    private Integer inWarehouseState;

    @ApiModelProperty(value = "订单出货状态(字典 order_shipment_state)")
    private Integer shipmentState;

    @ApiModelProperty(value = "订单到仓状态(字典 order_to_warehouse_state)")
    private Integer toWarehouseState;

    @ExcelProperty("发货人姓名/手机号")
    @ApiModelProperty(value = "发货人姓名/手机号")
    private String consignorNameOrPhone;

    @ExcelProperty("收货人姓名/手机号")
    @ApiModelProperty(value = "收货人姓名/手机号")
    private String consigneeNameOrPhone;

    @ExcelProperty("品名")
    @ApiModelProperty(value = "品名")
    private String title;

    @ExcelProperty("提单号")
    @ApiModelProperty(value = "提单号")
    private String tidanNo;

    @ApiModelProperty(value = "费用来源：1 订单计算 2 费用申请 3 特需费用")
    private Integer feeSource;

    @ApiModelProperty(value = "税点")
    private BigDecimal taxPoint;

    @ApiModelProperty(value = "含税金额")
    private BigDecimal taxAmount;
    @ApiModelProperty(value = "客户")
    private Long customerId;
    @ApiModelProperty(value = "目的国")
    private String destCountry;
    @ApiModelProperty(value = "自编号")
    private String containerNumber;
}
